clear all
set mem 500m
set more off

cd "C:\Users\Roy\Documents\My Dropbox\Tennis\Matlab\challenge_value\"

// SET THE PARAMETERS TO MATCH THE RAW VALUE/POLICY FILES YOU WANT TO CONVERT TO DTA FORMAT
local p_cc = 0.02

// This should be set according to what you set above
local p_cc_str = "p_cc_" + subinstr(string(`p_cc',"%3.2f"),".","p",.)
local states_3chal = 23928
local states_2chal = 18048

foreach numChals in 2 3 {
	local pref "valuesPolicies_`numChals'chals"
	// The values and policies were calculated in 5 chunks, each for a different value of p_win_if_serve (iterating over 5 p_win_if_receive).
	forvalues chunk = 1/5 {
		local file = "`pref'_`p_cc_str'_chunk`chunk'"
		di as input "Loading `file'.txt"
		
		insheet p_win_if_serve p_win_if_receive value p_mot tolerance_or_error dropme using `file'.txt, clear

		drop dropme
		
		foreach var of varlist value-tolerance_or_error {
			local type : type `var'
			if (substr("`type'", 1, 3) == "str") {
				replace `var' = "" if `var' == "NaN"
				replace `var' = "-99999999" if regexm(`var', "\-Inf")
				replace `var' = "99999999" if regexm(`var', "Inf")
			}
			destring `var', replace
		}

		gen state = mod(_n-1, `states_`numChals'chal')+1
		
		di "Merging states index"
		joinby state using stateIndex_`numChals'chals_per_set
		
		gen p_cc = `p_cc'
		label variable p_cc "Pr[close call]"

		drop state
		order value-tolerance_or_error, after(chals_left)
		order p_cc
		
		label variable p_win_if_serve "Pr[I win point | I am serving]"
		label variable p_win_if_receive "Pr[I win point | I am receiving]"
		label variable value "V(score,serving,challenges_left)"
		label variable p_mot "p_mot*(score,serving,challenges_left)"
		label variable tolerance_or_error "If deuce then tolerance level, else then orig p_mot (>1 or <0)"

		save `file'_work.dta, replace
	}


	clear
	set mem 800m

	forvalues chunk = 1/5 {
		local file = "`pref'_`p_cc_str'_chunk`chunk'_work"
		if (`chunk' == 1) {
			di as input "Appending all files, loading the first"
			use `file'
		}
		else {
			di "Appending `file'"
			append using `file'
		}
	}

	di as input "THIS SAVE COMMAND DOES NOT REPLACE THE FILE ON DISK AUTOMATICALLY!!! IF THERE IS ANOTHER FILE ALREADY, YOU NEED TO CHANGE THE NAME IN THE CODE (or delete the old file) - MAKE SURE YOU ARE SAVING IT UNDER A GOOD FILENAME!!!"
	save `pref'_`p_cc_str'.dta

	forvalues chunk = 1/5 {
		local file = "`pref'_`p_cc_str'_chunk`chunk'"
		cap erase `file'.txt
		cap erase `file'_work.dta
	}
}

/* This was for the time when we had more than one close-call probability,  one file for each p_cc. 
 (now we focus on p_cc=0.02, and divided the files between the p_win_point_serve)
// This file takes the raw (txt) files in valuesPoliciesRaw.rar, translates them to 
// dta files, and saves them in a new rar file valuesPoliciesReady.rar

! set path="C:\Program Files\WinRAR\";%path% & unrar e -o+ valuesPoliciesRaw.rar
local pref "valuesPolicies_3chals"


forvalues i = 1/20 {
	local p_cc = `i'/100
	local file = subinstr("`p_cc'", ".", "p", .)
	local file = "`pref'_p_cc_0`file'"
	di as input "Rendering `file'"
	insheet p_win_if_serve p_win_if_receive value p_mot tolerance_or_error dropme using `file'.txt, clear
	drop dropme
	
	foreach var of varlist value-tolerance_or_error {
		replace `var' = "" if `var' == "NaN"
		replace `var' = "-99999999" if regexm(`var', "\-Inf")
		replace `var' = "99999999" if regexm(`var', "Inf")
	}
	destring value-tolerance_or_error, replace

	gen state = mod(_n-1, 18048)+1
	
	di "Merging states index"
	joinby state using stateIndex
	
	gen p_cc = `p_cc'
	label variable p_cc "Pr[close call]"
	
	drop state
	order value-tolerance_or_error, after(chals_left)
	order p_cc
	
	label variable p_win_if_serve "Pr[I win point | I am serving]"
	label variable p_win_if_receive "Pr[I win point | I am receiving]"
	label variable value "V(score,serving,challenges_left)"
	label variable p_mot "p_mot*(score,serving,challenges_left)"
	label variable tolerance_or_error "If deuce then tolerance level, else then orig p_mot (>1 or <0)"
	
	save `file'_work.dta, replace
	erase `file'.txt
}

clear
set mem 800m

di "Appending all files, loading the first"
use valuesPolicies_p_cc_0p01_work
erase valuesPolicies_p_cc_0p01_work.dta
forvalues i = 2/20 {
	local p_cc = `i'/100
	local file = subinstr("`p_cc'", ".", "p", .)
	local file = "valuesPolicies_p_cc_0`file'_work"
	di "Appending `file'"
	append using `file'
	if (`i' != 2) {   // Keep the p_cc=0.02 intact for the regressions specific to that p_cc.
		erase `file'.dta
	}
}

save valuesPolicies.dta, replace

cap erase valuesPoliciesReady.rar
! set path="C:\Program Files\WinRAR\";%path% & rar a valuesPoliciesReady valuesPolicies.dta
erase valuesPolicies.dta
*/
