%% Calculate p_mot matrices for one p_cc.
% This is usually done by the first part, but if we want to run this separately
% for the simulation and probability to win the match calculation then we can
% run just this part

% CHANGE THE FILENAME AT THE END OF THE FUNCTION IF YOU'RE CHANGING SOMETHING
% AND DON'T WANT TO OVERWRITE.
function [p_mot_mega V_mega] = tryCalcValuesInChunksFunc(p_serve_i, calc_3chals_too, ...
                                                         p_cc, alpha, beta)
    strcat('p_serve_i of this chunk is:  ', num2str(p_serve_i))
    
    % Generally, we can create Paot(p) for any p~Beta(alpha,beta) distribution of p that
    % we wish. The formula is:
    %      Paot(p) = alpha/(alpha+beta)*[(1-betainc(p,alpha+1,beta))/(1-betainc(p,alpha,beta))]
    %   where betainc(x,y,z) is the CDF of x when x~Beta(y,z).
    % Specifically, if alpha=1, then we get a *** LINEAR *** Paot(p) with:
    % Paot(p) = 1/(1+beta)*[(1-betainc(p,2,beta))/(1-betainc(p,1,beta))]
    % so to get Paot(0) = 0.295 we need to set beta=2.39
    if (~exist('p_cc','var')) 
        p_cc = 0.02
    else
        p_cc
    end
    
    if (~exist('alpha','var'))
        alpha = 1
    else
        alpha
        if (~exist('beta','var'))
            beta  = alpha*(1/.295-1)
        else
            beta
        end
    end
    
    % The nonlinear specification we chose alpha=2, then we get Paot(0)=2/(2+?)
    % so to hit Paot(0)=0.295 we need beta=4.78:
%    alpha = 2
%    beta = 4.78

    % backward compatibility with an earlier version of the code (not in paper),
    % with linear Paot with Paot(0)=0.25
%    alpha = 1
 %   beta = 3
    
    p_aot_handle = @(p) p_aot(p,alpha,beta)
    G_handle = @(p) betainc(p,alpha,beta);

    % The original linear specification had 0.25 as the intercept, or Paot(0)=0.25.
    % This made the Beta distribution very easy to express as (?,?)=(1,3) are
    % integers. This way we got:
    % p_aot_handle = @(x) 0.25 + 0.75.*x;
    % G_handle = @(x) 1 - (1 - x).^3 ;
    % but then we generalized, so this comment is just for the record.

    p_win_serve_grid = .1:.2:.9;
    p_win_receive_grid = .1:.2:.9;

    p_mot_mega = cell(size(p_win_receive_grid, 2));
    V_mega = cell(size(p_win_receive_grid, 2));

    p_win_if_serve = p_win_serve_grid(p_serve_i)
    for challengesInSetRule = 2:3
        if (challengesInSetRule == 3 && exist('calc_3chals_too', 'var') && calc_3chals_too == false)
            continue
        end
        challengesInSetRule
        bigDB = [];
        for p_receive_i = 1:length(p_win_receive_grid)
            p_win_if_receive = p_win_receive_grid(p_receive_i)

            [V, p_mot, DB] = calcValues(p_win_if_serve, p_win_if_receive, p_cc, ...
                                    G_handle, p_aot_handle, challengesInSetRule);

            bigDB = [bigDB ; ...
                    [ones(size(DB,1),1)*[p_win_if_serve, p_win_if_receive] , DB]];

            if (challengesInSetRule == 2) 
                p_mot_mega{p_receive_i} = p_mot;
                V_mega{p_receive_i} = V;
            end
            
            if (V == -999)
                display('ERROR! Failure to converge in one of the deuces');
                display(p_cc);
                display(p_win_if_serve);
                display(p_win_if_receive);
            end
        end

        % CHANGE THE FILENAME IF YOU'RE CHANGING SOMETHING.
        filename = sprintf('valuesPolicies_%gchals_p_cc_%s_chunk%g.txt', challengesInSetRule, strrep(num2str(p_cc),'.','p'), p_serve_i);
        save(filename, 'bigDB', '-tabs', '-ascii');
    end
end

