% This function is based on the calcValues functions, but now we don't calculate
% p_mot's. We just plug in our policies (to allow for non-optimal policies) and
% see what happens to the value given those policies.

% V is the value matrix (indices are scores,serving,challenges), p_mot is a
% similar matrix for policy function and DB is a list of points (according to
% the order of score::goBackwards) with their V and p_mot listed.
function [V] = calcValuesFromKnownPmots(p_win_serve,p_win_receive,p_cc,... 
                         cdf_of_p_mot, p_aot_of_p_mot, p_mot_matrix, challengesInSetRule)  
                        % (above are the function handles)
    %% Initialize
    % Define the value and policy matrices
    V = NaN(77,77,2,4);
    for serve = 0:1
        V(scr2ind(score(2,0,0,0,0,0,serve,challengesInSetRule,challengesInSetRule))) = 1;
        V(scr2ind(score(2,0,0,1,0,0,serve,challengesInSetRule,challengesInSetRule))) = 1;
        V(scr2ind(score(1,0,0,2,0,0,serve,challengesInSetRule,challengesInSetRule))) = 0;
        V(scr2ind(score(0,0,0,2,0,0,serve,challengesInSetRule,challengesInSetRule))) = 0;
    end
    
    p_win_by_serve = [p_win_receive, p_win_serve]';
    
    p_win = kron(p_win_by_serve, ones(3,1));
    
    % choose  one of the scores from which if you'll go backwards you'll reach the 1.1 6:6 6-6
    % score (from which the calculation will start.
    s = score(2,0,0,0,0,0,0, challengesInSetRule, challengesInSetRule);
    
%    DB = NaN(18048, 1);
%    DB_index = 1;
    %% Start filling in the matrices
    % As long as we're not in 0.0 0:0 0-0:
    while ~isequal(goBackwards(s,1), s)
        s = goBackwards(s,1);
        if isdeuce(s) 
            s.challenges = 0;
%            display('Working on a deuce. Score is: ');
 %           show(s);

            chals = challengesInSetRule + istiebreak(s);
            
            % Start by calculating the values for the challenges=0 cells. They
            % do not depend on endogenous decisions because there are no
            % challenges left.
            % We have three scores (40-Adv, 40-40, Adv-40) and two serving
            % states, so we have 6 states for challenges=0. 
            % The two-column representation of the values is a 3X2 matrix of
            % values and the one-column representation of values is a 6X1 vector
            % (we need it for solving the system of equations)

            % calculating the values for challenges-left = 0
            % calculating the part before 
            M_exog = zeros(6);
            u_exog = zeros(6,1);
            for i=1:6
                temps = deuce2scr(s,i);
                % putting the A
                if scr2deuce(incrementMy(temps,1)) > 0 
                    M_exog(i,scr2deuce(incrementMy(temps,1))) = ...
                            p_win_by_serve(1 + temps.serving);
                else
                    u_exog(i) = p_win_by_serve(1 + temps.serving) * ...
                        V(scr2ind(incrementMy(temps, 1)));
                end
                
                % putting the B
                if scr2deuce(incrementOpp(temps,1,0)) > 0 
                    M_exog(i,scr2deuce(incrementOpp(temps,1,0))) = ...
                            1 - p_win_by_serve(1 + temps.serving);
                else
                    u_exog(i) = (1 - p_win_by_serve(1 + temps.serving)) * ...
                        V(scr2ind(incrementOpp(temps, 1, 0)));
                end
            end
            
            % calculate the values.
            v_exog = inv(eye(6)-M_exog)*u_exog;
            
            % updating the values in V.
            for i=1:6
                V(scr2ind(deuce2scr(s,i))) = v_exog(i);
%                DB(DB_index) = v_exog(i);
%                DB_index = DB_index + 1;
            end

            for cur_chal = 1:chals
                % In the case where we need to find p_mot we need to guess and
                % iterate. Now we get the p_mot's so we just need to calculate
                % the values.
                s.challenges = cur_chal;
                
                % Get the p_mot's from the matrix
                for i=1:6
                    p_mot_vec = max(min(1,p_mot_matrix(scr2ind(deuce2scr(s,i)))),0);
                end

                p_aot = p_aot_of_p_mot(p_mot_vec);
                G     = cdf_of_p_mot(p_mot_vec);

                As = p_win + ((1-p_win) .* p_cc) .* (1 - G) .* p_aot ;
                Bs = (1 - p_win) .* (p_cc*G + (1 - p_cc)) ;
                Cs = ((1 - p_win) * p_cc) .* (1 - G) .* (1 - p_aot);

                M = zeros(6);
                u = zeros(6,1);

                for i=1:6
                    temps = deuce2scr(s,i);

                    % putting the A
                    if scr2deuce(incrementMy(temps,1)) > 0   % if endogenous value
                        M(i,scr2deuce(incrementMy(temps,1))) = As(i);
                    else                                     % if exogenous value
                        u(i) = As(i)*V(scr2ind(incrementMy(temps,1)));
                    end

                    % putting the B
                    if scr2deuce(incrementOpp(temps,1,0)) > 0 % if endogenous value
                        M(i,scr2deuce(incrementOpp(temps,1,0))) = Bs(i);
                    else                                      % if exogenous value
                        u(i) = Bs(i)*V(scr2ind(incrementOpp(temps,1,0)));
                    end

                    % putting the C (value exogenous always b/c calculated in previous round)
                    u(i) = u(i) + Cs(i)*V(scr2ind(incrementOpp(temps,1,1)));
                end

                % ready to calculate this iteration's v:
                v_next = inv(eye(size(M))-M)*u;                

                for i=1:6
                    V(scr2ind(deuce2scr(s,i))) = v_next(i);
%                    DB(DB_index) = v_next(i);
%                    DB_index = DB_index + 1;
                end
            end
        else
            % In a regular point we have the values of the following points, so
            % we can immediately calculate p_mot and V
            % For all challenges from 2 down to 0 or from 3 down to 0 (depending
            % on whether it's a tiebreak).
            for servstat = 0:1 
                cur_p_win = p_win_by_serve(servstat + 1);
                s.serving = servstat;
                
                % for challenges from 2 (or 3) to 1 (challenge 0 will be dealt
                % with later
                for chals = (challengesInSetRule+istiebreak(s)):-1:1
                    s.challenges = chals;
                    
                    cur_p_mot = max(min(p_mot_matrix(scr2ind(s)),1),0);
                    
                    p_aot = p_aot_of_p_mot(cur_p_mot);
                    G     = cdf_of_p_mot(cur_p_mot);
                
                    A = cur_p_win + (1-cur_p_win) * p_cc * (1 - G) * p_aot;
                    B = (1 - cur_p_win) * (p_cc*G + (1 - p_cc));
                    C = (1 - cur_p_win) * p_cc * (1 - G) * (1 - p_aot);
                    
                    V(scr2ind(s)) = A * V(scr2ind(incrementMy(s,1))) + ...
                                    B * V(scr2ind(incrementOpp(s,1,0))) + ...
                                    C * V(scr2ind(incrementOpp(s,1,1)));
                    
                     if (~isreal(V(scr2ind(s))) || ~isreal(cur_p_mot)) 
                         display('NON REAL VALUE OR POLICY!!');
                         display(v_next);
                         display(cur_p_mot);
                         show(s);
                     end
%                    DB(DB_index) = V(scr2ind(s));
%                    DB_index = DB_index + 1;
                end
                
                s.challenges = 0;
                V(scr2ind(s)) = cur_p_win * V(scr2ind(incrementMy(s,1))) + ...
                                (1 - cur_p_win) * V(scr2ind(incrementOpp(s,1,0)));
%                DB(DB_index) = V(scr2ind(s));
%                DB_index = DB_index + 1;
            end
        end
    end
end