function [p_aot_in_data_handleNLS, figuresTable] = ...
            compareData2Optimal(myDataset, optimalPolicy, F_alpha, F_beta_model)

    % Interpolate policy at the data points from the optimal policy cell array (grid)
    policy_interpolated = NaN(size(myDataset.challenge_id,1),1);
    
    [X, Y] = meshgrid(.1:.2:.9,.1:.2:.9);
    % Yes, this will be a bit ugly with the loop (nice to read, but probably
    % very slow).
    for i=1:size(myDataset.challenge_id,1)
        thisRow = myDataset(i,:);
        thisState = score(  thisRow.mysets, ...
                            thisRow.mygames, ...
                            thisRow.mypoints, ...
                            thisRow.oppsets, ...
                            thisRow.oppgames, ...
                            thisRow.opppoints, ...
                            thisRow.serving, ...
                            thisRow.chals_left, ...
                            thisRow.chals_in_set_rule);
                            
        Z = cellfun(@(A) A(scr2ind(thisState)), ...
                optimalPolicy{thisRow.chals_in_set_rule-1});
        
        policy_interpolated(i) = interp2(X,Y,Z,...
                                         min(max(thisRow.p_win_pt_receive,.1),.9), ...
                                         min(max(thisRow.p_win_pt_serve,.1),.9));
    end
    
    % estimate F_beta in the data, given the F_alpha, by minimizing the sum of 
    % squared residuals:
    sample4reg = ~isnan(policy_interpolated);
    display(sprintf('There are %g observations with missing interpolated policies', ...
                    size(policy_interpolated)-sum(sample4reg)));
                
    [estimatedBetaNLS, ssr, flag] = ...
        fminsearch(@(beta_prm) ...
        (myDataset.challenge_won(sample4reg)-p_aot(policy_interpolated(sample4reg),F_alpha,beta_prm))'*...
        (myDataset.challenge_won(sample4reg)-p_aot(policy_interpolated(sample4reg),F_alpha,beta_prm)),0)
    
    p_aot_in_data_handleNLS = @(p) p_aot(p, F_alpha, estimatedBetaNLS);
    p_aot_model_handle = @(p) p_aot(p, F_alpha, F_beta_model);
   
    display('implied intercept from NLS is...');
    1/(F_alpha+estimatedBetaNLS)
    
    % Regress challenge_won on the interpolated p_mot (linearly, OLS, but
    % restrict the line to pass by (1,1))
    estimated_intercept = 1-regress(myDataset.challenge_won-1,policy_interpolated-1);
    display('intercept from (constrained) linear regression is...');
    estimated_intercept
    
    p_aot_in_data_handle_OLSrestricted = @(p) [ones(size(p)), p]*[estimated_intercept;... 
                                                                  (1-estimated_intercept)];
    
    free_prms_lin = regress(myDataset.challenge_won, ...
                            [ones(size(policy_interpolated)),policy_interpolated]);
    p_aot_in_data_handle_OLSfree = @(p) [ones(size(p)), p]*free_prms_lin;
    
    figuresTable = dataset( { [ (0:0.01:1)' , ...
                                ksdensity(policy_interpolated,0:0.01:1)' , ...
                                p_aot_model_handle(0:0.01:1)', ...
                                p_aot_in_data_handleNLS(0:0.01:1)' , ...
                                p_aot_in_data_handle_OLSrestricted((0:0.01:1)'), ...
                                p_aot_in_data_handle_OLSfree((0:0.01:1)')], ...
                             'p_mot','kdensity','p_aot_model','p_aot_data_NLS',...
                             'p_aot_data_OLS_restricted','p_aot_data_OLS_free'});
end