clear all
% This program plots the pdf of the distribution of p, the marginal probability,
% then it plots the Paot(p) relationship (Paot(p) is G(p) in the paper)

% rename the beta function (because we'll name the parameter beta with the same
% name: beta).
betafunc = @(x,y) beta(x,y)

% The space of p's
p = linspace(0,1,200);

% This is Paot(0) and we want to match it to the data.
intercept = 0.295

% compute the pdfs...
% (1) Linear...
alpha = 1;
beta = alpha*(1/intercept-1);  % To get Paot(0)=intercept... 
pdf_lin = 1/betafunc(alpha,beta) * (p.^(alpha-1)) .* ((1-p).^(beta-1));
Paot_lin = (alpha/(alpha+beta)).*((1-betainc(p,alpha+1,beta))./(1-betainc(p,alpha,beta)));

% (2) Convex
alpha = 2;
beta = alpha*(1/intercept-1);
pdf_convex = 1/betafunc(alpha,beta) * (p.^(alpha-1)) .* ((1-p).^(beta-1));
Paot_convex = (alpha/(alpha+beta)).*((1-betainc(p,alpha+1,beta))./(1-betainc(p,alpha,beta)));

% (3) Concave
alpha = .5;
beta = alpha*(1/intercept-1);
pdf_concave = 1/betafunc(alpha,beta) * (p.^(alpha-1)) .* ((1-p).^(beta-1));
Paot_concave = (alpha/(alpha+beta)).*((1-betainc(p,alpha+1,beta))./(1-betainc(p,alpha,beta)));

% plot pdf
plot(p, pdf_lin,p,pdf_convex,p,pdf_concave);
title('PDF of probability to overturn a decision, before seeing the ball, conditional on being a close call');
xlabel ('p');
ylabel ('f(p|p>0)');
legend(strcat('p~Beta(1,',num2str(roundn(1/intercept-1,-2)),')'), ...
        strcat('p~Beta(2,',num2str(roundn(2*(1/intercept-1),-2)),')'), ...
        strcat('p~Beta(0.5,',num2str(roundn(.5*(1/intercept-1),-2)),')'))
saveas(gca,'pdfs.png');

% plot Paot
plot(p, Paot_lin,p,Paot_convex,p,Paot_concave, p, p);
title('Average overturning probability: $G(p^\star)$, or $E\left[p|p>p^\star\right]$','interpreter','latex');
xlabel ('p^*');
ylabel ('G(p^*)');
legend(strcat('p~Beta(1,',num2str(roundn(1/intercept-1,-2)),')'), ...
        strcat('p~Beta(2,',num2str(roundn(2*(1/intercept-1),-2)),')'), ...
        strcat('p~Beta(0.5,',num2str(roundn(.5*(1/intercept-1),-2)),')'), ...
        '45-degrees line', 'Location', 'Southeast')
saveas(gca,'Paots.png');

