% This program runs the robustness checks. 
% For a small grid of parameters, we
% run the model, estimate what's in the data in comparison to the model, and
% calculate values of alternative suboptimal strategies.
%
% To run the main specification for table (and figure) 4, run
% trySimulateMatchFunc('<filename_suffix>',1,0.02,0.295)
% 
%% Initializations
F_alphas   = [.5 , 1 , 2];
p_ccs      = [.01 ,.02, .03];
intercepts = [.25 , .295, .35];

%% Robustness Checks Run
counter = 0;
for F_alpha = F_alphas
    for p_cc = p_ccs
        for intercept = intercepts
            filename = strrep(sprintf('F_alph%g_pcc%g_intrcpt%g',F_alpha,p_cc,intercept),'.','p');
            command = strcat('matlab -nodesktop -r', ...
                          ' "global suppress_warnings;suppress_warnings=1;',...
                          'cd ''C:\Users\Roy\Documents\My Dropbox\Tennis\Matlab\challenge_value'';',...
                          sprintf('trySimulateMatchFunc(''%s'',%g,%g,%g);exit;"', filename,p_cc,F_alpha,intercept));

            % run the programs in batches of 7 (not to make the computer go
            % nuts)
            if (mod(counter+1,6) > 0)
                command = strcat(command, ' &');
            end
            
            command
            
            system(command);

            counter = counter+1;
        end
    end
end

%% Unite the results to one table of average efficiency
grandTotal = NaN(27,6);
counter = 1;
for F_alpha = F_alphas
    for p_cc = p_ccs
        for intercept = intercepts
            filename = strrep(sprintf('F_alph%g_pcc%g_intrcpt%g',F_alpha,p_cc,intercept),'.','p');
            load(strcat('opt_subopt_cmp',filename));
            load(strcat('figures_data_',filename));

            % Weight the distance between the model's G and the G estimated from
            % the data.
            weighted_vert_diff = sum(figuresTable.kdensity(:,1))^-1 * figuresTable.kdensity' * ...
                    (figuresTable.p_aot_data_NLS-figuresTable.p_aot_model);
                    
            grandTotal(counter,:) = [F_alpha, p_cc, intercept, ...
                                        outputTable(26,8:9), weighted_vert_diff];
            counter = counter+1;
        end
    end
end