classdef score
    % properties are the sets, games and points by each side
    properties % (SetAccess = private, GetAccess = private) <== private is MATLAB 2008 and later 
        mySets=0
        myGames=0
        myPoints=0
        oppSets=0
        oppGames=0
        oppPoints=0
        serving=1
        challengesInSetRule=2
        challenges=challengesInSetRule
    end
    
    methods
        function obj=score( mySets,     myGames,    myPoints, ...
                            oppSets,    oppGames,   oppPoints, serving, ...
                            challenges, challengesInSetRule)
            obj.mySets    = mySets;
            obj.myGames   = myGames;
            obj.myPoints  = myPoints;
            obj.oppSets   = oppSets;
            obj.oppGames  = oppGames;
            obj.oppPoints = oppPoints;
            obj.serving   = serving;
            obj.challenges = challenges;
            obj.challengesInSetRule = challengesInSetRule;
        end
        function show(obj)
            if obj.serving 
                servestr = 'serving';
            else
                servestr = 'receiving';
            end
            
            if (obj.myGames ~= 6 || obj.oppGames ~= 6) 
                gameScore = {'0' , '15', '30', '40', 'Adv'};            
                display(sprintf('%g-%g (sets) %g:%g (games) %s:%s (points), %g challenges left, %s\n', ...
                            obj.mySets  , obj.oppSets, ...
                            obj.myGames , obj.oppGames, ...
                            gameScore{obj.myPoints+1}, ...
                            gameScore{obj.oppPoints+1}, ...
                            obj.challenges, ...
                            servestr));

            else
                display(sprintf('%g-%g (sets) %g:%g (games) %g:%g (points), %g challenges left, %s\n', ...
                            obj.mySets  , obj.oppSets, ...
                            obj.myGames , obj.oppGames, ...
                            obj.myPoints, obj.oppPoints, ...
                            obj.challenges, ...
                            servestr));
            end
        end
        % check if the score is a valid score
        % i.e if we're supposed to play the following point or not
        function valid = validate(obj)
            % if something is negative
            if min([obj.mySets, obj.myGames, obj.myPoints, obj.oppSets, ...
                    obj.oppGames, obj.oppPoints, obj.challenges]) < 0
                valid = 0;
                return
            end
            % if sets are invalid
            if max(obj.mySets, obj.myGames) > 1
                valid = 0;
                return
            end
            % if games are invalid
            if (max(obj.myGames, obj.oppGames) > 6) || ...
                    (max(obj.myGames, obj.oppGames) == 6 && ...
                     min(obj.myGames, obj.oppGames) < 5) 
                valid = 0;
                return
            end
            % if points are invalid
            if (obj.myGames == 6 && obj.oppGames == 6)
                endson = 7;
            else
                endson = 4;
            end
            if (max(obj.myPoints, obj.oppPoints) >= endson) && ...
                    (abs(obj.myPoints - obj.oppPoints) >= 2)
                valid = 0;
                return;
            end
            if (endson == 7 && obj.challenges > (obj.challengesInSetRule + 1) || ... 
                    endson == 4 && obj.challenges > obj.challengesInSetRule)
                valid = 0;
                return;
            end
                
            valid = 1;
        end
        
        % incrementMy updates the match score assuming I won the point.
        % increments is the number of points you want to advance the score. It
        % must be a positive integer.
        function newobj = incrementMy(obj, increments)
            mySets1    = obj.mySets;
            oppSets1   = obj.oppSets;
            myGames1   = obj.myGames;
            oppGames1  = obj.oppGames;
            myPoints1  = obj.myPoints;
            oppPoints1 = obj.oppPoints;
            serving1   = obj.serving;
            challenges1 = obj.challenges;
            
            for i=1:increments
                myPoints1 = myPoints1 + 1;  % increment point
                % check if a game/set was won
                % first, check if this is a tiebreaker.
                if (myGames1 == 6 && oppGames1 == 6)
                    endson = 7;
                    % in a tiebreak, serving changes after points:
                    % 1, 0, 1, 0, 1 ...
                    % since I already incremented myPoints1, we need mod(1+0,2)
                    % to be 1 and not 0 when we change.
                    if (mod(myPoints1 + oppPoints1, 2) == 1) 
                        serving1 = 1 - serving1;
                    end
                else
                    endson = 4;
                end
                
                % if it's deuce and 4-4 then set it to be 3-3
                % (same for 7-7 in tiebreak)
                if (oppPoints1 == endson && myPoints1 == endson)
                    oppPoints1 = endson - 1;
                    myPoints1  = endson - 1;
                end
                
                % if we're going to the next game
                if (max(myPoints1, oppPoints1) >= endson) && ...
                        (abs(myPoints1 - oppPoints1) >= 2)
                    myGames1  = myGames1 + 1;   % we know it's *my* game because
                                                % we're increasing *my* points.
                    % if it was a tiebreak then serving1 should be the
                    % opposite of serving1 in the *beginning* of the tiebreak.
                    % however, this might not be the serving1 in the last point.
                    % we already changed serves if (myPoints1 + oppPoints1) is
                    % odd so we'll undo that, and then the serving identity goes
                    % like: 1 , 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, ...
                    % so for mod(myPoints1 + oppPoints1, 4) == 2 or 3 we have 1
                    % (and should switch serves)
                    if (myGames1 == 7 && oppGames1 == 6)
                        % undo what we did before at the point level
                        if (mod(myPoints1 + oppPoints1, 2) == 1) 
                            serving1 = 1 - serving1;
                        end
                        % decide who starts next set.
                        if (mod(myPoints1 + oppPoints1 - 2, 4) > 1) 
                            serving1 = 1 - serving1;
                        end
                    % If it was a regular game, simply switch serves.
                    else
                        serving1 = 1 - serving1;                      
                    end
                    % if we entered to a tiebreak, add a challenge
                    if (myGames1 == 6 && oppGames1 == 6)
                        challenges1 = challenges1 + 1;
                    end
                    myPoints1  = 0;
                    oppPoints1 = 0;
                    if (myGames1 == 6 && oppGames1 <= 4) || ...
                        (myGames1 == 7)
                        myGames1  = 0;
                        oppGames1 = 0;
                        mySets1  = mySets1 + 1;
                        % new set => reset challenges to the number of
                        % challenges they have every new set
                        challenges1 = obj.challengesInSetRule;
                    end
                end
            end
            newobj = score(mySets1, myGames1, myPoints1, oppSets1, oppGames1, ...
                            oppPoints1, serving1, challenges1, obj.challengesInSetRule);
        end

        % incrementOpp updates the match score assuming the opponent won the 
        % point. increments is the number of points you want to advance the 
        % score for the opponent (positive integer). challenges is the number of
        % challenges left reduced to me (nonnegative integer).
        % First, the remove_challenges will be removed and then challenges 
        % will updated according to the score advancing (increased by 1 when 
        % tiebreak starts and reset to 2 when the next set starts).
        function newobj = incrementOpp(obj, increments, remove_challenges)
            mySets1    = obj.mySets;
            oppSets1   = obj.oppSets;
            myGames1   = obj.myGames;
            oppGames1  = obj.oppGames;
            myPoints1  = obj.myPoints;  
            oppPoints1 = obj.oppPoints; 
            serving1   = obj.serving;
            challenges1 = max(obj.challenges - remove_challenges, 0);

            for i=1:increments
                oppPoints1 = oppPoints1 + 1;  % increment point

                % check if a game/set was won
                % first, check if this is a tiebreaker.
                if (myGames1 == 6 && oppGames1 == 6)
                    endson = 7;
                    % in a tiebreak, serving changes after points:
                    % 1, 0, 1, 0, 1 ...
                    % since I already incremented myPoints1, we need mod(1+0,2)
                    % to be 1 and not 0 when we change.
                    if (mod(myPoints1 + oppPoints1, 2) == 1) 
                        serving1 = 1 - serving1;
                    end
                else
                    endson = 4;
                end

                % if it's deuce and 4-4 then set it to be 3-3
                % (same for 7-7 in tiebreak)
                if (oppPoints1 == endson && myPoints1 == endson)
                    oppPoints1 = endson - 1;
                    myPoints1  = endson - 1;
                end

                % if we're going to the next game
                if (max(myPoints1, oppPoints1) >= endson) && ...
                        (abs(myPoints1 - oppPoints1) >= 2)
                    oppGames1  = oppGames1 + 1;   % we know it's *opp*'s game because
                                                  % we're increasing *opp* points.
                                        % if it was a tiebreak then serving1 should be the
                    % opposite of serving1 in the *beginning* of the tiebreak.
                    % however, this might not be the serving1 in the last point.
                    % we already changed serves if (myPoints1 + oppPoints1) is
                    % odd so we'll undo that, and then the serving identity goes
                    % like: 1 , 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, ...
                    % so for mod(myPoints1 + oppPoints1, 4) == 2 or 3 we have 1
                    % (and should switch serves)
                    if (myGames1 == 7 && oppGames1 == 6)
                        % undo what we did before at the point level
                        if (mod(myPoints1 + oppPoints1, 2) == 1) 
                            serving1 = 1 - serving1;
                        end
                        % decide who starts next set.
                        if (mod(myPoints1 + oppPoints1 - 2, 4) > 1) 
                            serving1 = 1 - serving1;
                        end
                    % If it was a regular game, simply switch serves.
                    else
                        serving1 = 1 - serving1;               
                    end
                    
                    % if we entered to a tiebreak, add a challenge
                    if (myGames1 == 6 && oppGames1 == 6)
                        challenges1 = challenges1 + 1;
                    end

                    myPoints1  = 0;
                    oppPoints1 = 0;

                    if (oppGames1 == 6 && myGames1 <= 4) || ...
                        (oppGames1 == 7)
                        myGames1  = 0;
                        oppGames1 = 0;
                        oppSets1  = oppSets1 + 1;
                        
                        % new set => reset challenges to the number of
                        % challenges they have every new set
                        challenges1 = obj.challengesInSetRule;
                    end
                end
            end
            newobj = score(mySets1, myGames1, myPoints1, oppSets1, oppGames1, ...
                            oppPoints1, serving1, challenges1, obj.challengesInSetRule);
        end
        
        
        % We're going to have functions that map scores (and other stuff) 
        % to values or policies. Those functions will be represented by 
        % k-dimensional matrices with the first two dimensions my and my oppon'
        % scores. We thus need to translate the set,game,point tuple to a single
        % integer.
        % every regular game has 5 (from 0 to 4) points, and the sixth game
        % can potentially (if the other player also has six games) have 7 points
        % thus, every set has at most 6*5 + 8 = 38 points.
        % Thus, the whole matrix has 38*2=76 rows and 76 columns.
        % For the sake of terminal cells (those in which the game is over), I
        % add another row and column to represent scores such as 0-2 0:0 0-0
        % in which we don't play, but we know the opponent won (2 sets over 0).
        % Note that not all cells in the matrix represent valid scores.
        function index = scoreIndexMe(obj)
            index = int16(obj.mySets * 38 + obj.myGames * 5 + obj.myPoints + 1);
        end
        function index = scoreIndexOpp(obj)
            index = int16(obj.oppSets * 38 + obj.oppGames * 5 + obj.oppPoints + 1);
        end
        
        % instead of getting my and my opponent's score indexes separately, 
        % we can return a logical matrix (mask) with 1 in the cell that relates 
        % to the (s,c,srv) in question. c is outside this object, but will be
        % needed by the value calculation algorithm, so we will get it as an
        % argument. s and srv are part of the score object.
        % I am assuming the value matrix is 77 X 77 X 2 X (challenges in
        % tiebreak if you haven't lost any challenge + 1 = challenges in the
        % beginning of the set + 2)
        function mask = scr2ind(obj)
            mask = zeros(77,77,2,obj.challengesInSetRule+2);
            mask(scoreIndexMe(obj),scoreIndexOpp(obj),obj.serving + 1, ...
                obj.challenges + 1) = 1;
            mask = logical(mask);
        end
        
%         % the following function assing an integer index for each deuce cell
%         % starting from the deuce point of obj. We have 3 scores, 3 (non-
%         % tiebreak) or 4 (tiebreak) challenges states, and 2 serving states. 
%         % Thus, we have a 3X3X2 = 18 or 3X4X2 = 24 cells in the v vector we want
%         % to calculate. From the incrementMy and incrementOpp functions we will 
%         % be able to point from the LHS v vector to the RHS v vector.
%         function index = scr2deuce_old(obj)
%             if isdeuce(obj)
%                 % adv_index will be 0 for 40-Adv, 1 for 40-40 and 2 for Adv-40.
%                 adv_index = obj.myPoints - obj.oppPoints + 1;
%                 index = adv_index * (3 + istiebreak(obj)) * 2 + ...
%                         obj.challenges * 2 + obj.serving + 1;
%             else
%             	index = 0;
%             end
%         end
%         
%         % the following function will return the score of the index of the v
%         % vector of the deuce. Since each deuce has the same vector, we need the
%         % score of the deuce to know where we are in the big array.
%         function newobj = deuce2scr_old(obj, index)
%             if isdeuce(obj) && index >= 1 && index <= 6 * (3 + istiebreak(obj)) * 2
%                 serving     = mod(index - 1, 2);
%                 
%                 newindex    = floor((index - 1) / 2);
%                 challenges  = mod(newindex, 3 + istiebreak(obj));
%                 
%                 newindex    = floor(newindex / 3);
%                 deucebase   = 3 + istiebreak(obj) * 3;
%                 myPoints    = deucebase + (newindex == 2);
%                 oppPoints   = deucebase + (newindex == 0);
%                 
%                 newobj = score(obj.mySets, obj.myGames, myPoints, ...
%                                 obj.oppSets, obj.oppGames, oppPoints, ...
%                                 serving, challenges);
%             else
%                 newobj = NaN;
%             end
%         end 
        
        % the following function assing an integer index for each deuce cell
        % serving, score    =>   index
        % 0         40-Adv          1
        % 0         40-40           2
        % 0         Adv-40          3
        % 1         40-Adv          4
        % 1         40-40           5
        % 1         Adv-40          6
        function index = scr2deuce(obj)
            if isdeuce(obj)
                index = obj.serving * 3 + obj.myPoints - obj.oppPoints + 2;
            else
            	index = 0;
            end
        end
        
        % the following function will return the score of the index of the v
        % vector (an opposite mappping)
        function newobj = deuce2scr(obj, index)
            if isdeuce(obj) && index >= 1 && index <= 6
                scr_ind = mod(index - 1, 3);
                
                deucebase   = 3 + istiebreak(obj) * 3;
                myPoints    = deucebase + (scr_ind == 2);
                oppPoints   = deucebase + (scr_ind == 0);

                serving = floor((index - 1)/3);
                
                newobj = score(obj.mySets, obj.myGames, myPoints, ...
                                obj.oppSets, obj.oppGames, oppPoints, ...
                                serving, obj.challenges, obj.challengesInSetRule);
            else
                newobj = NaN;
            end
        end

        % This function will be used for the backwards induction algorithm.
        % It will start with the terminal nodes' values and go back.
        % Challenges are NOT updated when going backwards (the algorithm will
        % need to calculate the values for all challenges in any case).
        function newobj = goBackwards(obj, decrements)
            mySets1    = obj.mySets;
            oppSets1   = obj.oppSets;
            myGames1   = obj.myGames;
            oppGames1  = obj.oppGames;
            myPoints1  = obj.myPoints;
            oppPoints1 = obj.oppPoints;
            serving1   = obj.serving;
            
            for i=1:decrements
                if max(myPoints1, oppPoints1) == 0
                    if max(myGames1, oppGames1) == 0
                        if max(mySets1, oppSets1) == 0
                            % if it's 0.0 0:0 0-0 then there is no backwards
                            break;
                            
                        % if mySets1 <= oppSets1 decrease mySets1 more and
                        % more until you get to 0.1 or 0.2
                        elseif mySets1 <= oppSets1
                            if mySets1 > 0
                                mySets1 = mySets1 - 1;
                            % when we reach 0.1 or 0.2 we go to 1.0 and 2.1
                            % respectively (we won't have 0.2 and 2.1
                            % really)
                            else
                                mySets1 = oppSets1;
                                oppSets1 = oppSets1 - 1;
                            end

                            % if mySets1 > oppSets1 then we're in the branch
                            % that decreases oppSets only, until we get to x.0
                        else
                            if oppSets1 > 0
                                oppSets1 = oppSets1 - 1;
                            % when we get to x.0 in sets, we should move to the
                            % next "layer". that is, (x-1).(x-1)
                            else
                                mySets1  = mySets1 - 1;
                                oppSets1 = mySets1;
                            end

                        end
                        myGames1   = 6; % next (previous) set will start 
                        oppGames1  = 6; % in the end (i.e tiebreak 
                        myPoints1  = 6; % deuce)
                        oppPoints1 = 6;
                        
                        % when we go to the previous set's tiebreak from this
                        % set's first game we know that the server changes in
                        % the 6-6 point score because it's the same one who
                        % started the tiebreak: (mod(6+6-1,4)==mod(0+0-1,4)==3)
                        serving1 = 1 - serving1;
                        
                    % if the games are not 0:0 then we're still in the same
                    % set tree and we should go down and decrease myGames1
                    % one by one until we reach 0:x, then do x:(x-1) and
                    % remove opponent's games until we reach x:0 and then do
                    % (x-1):(x-1) and start over
                    % the only thing to be careful is not to move from 6:5
                    % to 6:4 (because 6:4 isn't really a game).
                    else
                        % remember the previous game score so that we'll know if
                        % the new game score has the same server as this or a
                        % different one.
                        prevMyGames = myGames1;
                        prevOppGames = oppGames1;
                        if myGames1 <= oppGames1
                            if (myGames1 > 0 && (oppGames1 < 6 || myGames1 > 5))
                                myGames1 = myGames1 - 1;
                            else
                                myGames1  = oppGames1;
                                oppGames1 = oppGames1 - 1;
                            end
                        else
                            if (oppGames1 > 0 && (myGames1 < 6 || oppGames1 > 5))
                                oppGames1 = oppGames1 - 1;
                            else
                                myGames1  = myGames1 - 1;
                                oppGames1 = myGames1;
                            end
                        end
                        
                        if (mod(prevMyGames + prevOppGames, 2) ~= ...
                                mod(myGames1 + oppGames1, 2))
                            serving1 = 1-serving1;
                        end
                        myPoints1  = 3; % prev game starts (ends) with a deuce
                        oppPoints1 = 3;
                        
                    end

                % same for points. if the game is not over, first take off my
                % points, then my opponents, then go back a layer.
                % the only thing to be careful is not to move from Adv:40
                % to Adv:30 (because Adv:30 isn't really a point).
                % this should not be a problem if we always follow this
                % goBackwards function because we should never get to a point of
                % Adv:40 or 40:Adv (the value calculation for those points will 
                % be carried out jointly with 40:40 so we don't need to arrive
                % this cell), but to have a good function that serves for other
                % purposes, I deal with this too. Nevertheless, the serving
                % update for those cells might be wrong.
                else
                    if (myGames1 == 6 && oppGames1 == 6)
                        endson = 7;
                    else
                        endson = 4;
                    end
                    prevMyPoints = myPoints1;
                    prevOppPoints = oppPoints1;
                    if myPoints1 <= oppPoints1
                        if (myPoints1 > 0 && ...
                                (oppPoints1 < endson || myPoints1 > endson-1))
                            myPoints1 = myPoints1 - 1;
                        else
                            myPoints1 = oppPoints1;
                            oppPoints1 = oppPoints1 - 1;
                        end
                    else
                        if (oppPoints1 > 0 && ...
                                (myPoints1 < endson || oppPoints1 > endson-1))
                            oppPoints1 = oppPoints1 - 1;
                        else
                            myPoints1  = myPoints1 - 1;
                            oppPoints1 = myPoints1;
                        end
                    end
                    % if it's a tiebreak and we're not moving to the previous
                    % game, then we should change serving if the original
                    % point's server is the same as the new point server.
                    % The serving change will not work for 7-6 and 6-7, but as
                    % written before, we should not get to those cells unless we
                    % start with them and there's no reason to start with them.
                    if (endson == 7)
                        if (floor(mod(prevMyPoints + prevOppPoints - 1, 4) / 2) ~= ...
                            floor(mod(myPoints1 + oppPoints1 - 1, 4) / 2))
                            serving1 = 1 - serving1;
                        end
                    end
                end
            end
            newobj = score( mySets1,    myGames1,   myPoints1, ...
                            oppSets1,   oppGames1,  oppPoints1, serving1, ...
                            obj.challenges, obj.challengesInSetRule);
                            % challenges here don't really have a meaning
                            % because we're going to enumerate on them anyways.
        end
        
        % This function says whether a score is a deuce or not
        % (assuming the score is valid, of course)
        function deuceflag = isdeuce(obj)
            if (obj.myGames == 6 && obj.oppGames == 6)
                endson = 7;
            else
                endson = 4;
            end
            if min(obj.myPoints, obj.oppPoints) == endson - 1
                deuceflag = true;
            else
                deuceflag = false;
            end
        end
        
        % This function says whether a score is in a tiebreak or not
        function tiebreakflag = istiebreak(obj)
            if (obj.myGames == 6 && obj.oppGames == 6)
                tiebreakflag = 1;
            else
                tiebreakflag = 0;
            end
        end
%         function obj = set.challenges(obj, value)
%             if (value >= 0 && value <= 2 + istiebreak(obj))
%                 obj.challenges = value;
%             else
%                 display(['Error while trying to set challenges left = ' ...
%                         value]);
%             end
%         end
%         function value = get.challenges(obj)
%             value = obj.challenges;
%         end
%         function obj = set.serving(obj, value)
%             if (value == 0 || value == 1)
%                 obj.serving = value;
%             else
%                 display('Error: serving must be either 0 or 1');
%             end
%         end
    end
end
