% This function calculates the OPTIMAL value function and policy for given
% technological assumptions.
% The function returns two cell arrays: one for the policy and one for the
% values (for each state). Each array has two cells (inside each of which there 
% is a 5X5 cell array): one for 2-challenges in a set, and another for a
% 3-challenges in a set rule.
% Within the 2- or 3-challenges rule cell there is a 5X5 array, representing the
% 5X5 grid points of the p_win_point_if_serve (or receive). Within each cell of
% the 5X5 (sub)array there is a matrix of states in a match and their
% corresponding value or policy.
% Thus, to reach the value of the state in S (given a 3-challenge-rule match in 
% which the challenger has 0.7 prob to win a point when serving and 0.5 prob of
% winning when receiving, one should look for
% V_mega{2}{4,3}(scr2ind(S))

function [p_mot_mega2 V_mega2] = tryCalcValuesFunc(calc_3chals_too, p_cc, alpha, beta)
    % Generally, we can create Paot(p) for any p~Beta(alpha,beta) distribution of p that
    % we wish. The formula is:
    %      Paot(p) = alpha/(alpha+beta)*[(1-betainc(p,alpha+1,beta))/(1-betainc(p,alpha,beta))]
    %   where betainc(x,y,z) is the CDF of x when x~Beta(y,z).
    % Specifically, if alpha=1, then we get a *** LINEAR *** Paot(p) with:
    % Paot(p) = 1/(1+beta)*[(1-betainc(p,2,beta))/(1-betainc(p,1,beta))]
    % so to get Paot(0) = 0.295 we need to set beta=2.39
    p_aot_handle = @(p) p_aot(p,alpha,beta)
    G_handle = @(p) betainc(p,alpha,beta);

    p_win_serve_grid = .1:.2:.9;
    p_win_receive_grid = .1:.2:.9;

    % p_mot_mega and 
    p_mot_mega2 = cell(1,2);
    V_mega2     = cell(1,2);

    for challengesInSetRule = 2:3
        if (challengesInSetRule == 3 && exist('calc_3chals_too', 'var') && calc_3chals_too == false)
            continue
        end
        challengesInSetRule
        p_mot_mega = cell(size(p_win_receive_grid, 2));
        V_mega = cell(size(p_win_receive_grid, 2));

        for p_serve_i = 1:length(p_win_serve_grid)
            p_win_if_serve = p_win_receive_grid(p_serve_i)
            
            for p_receive_i = 1:length(p_win_receive_grid)
                p_win_if_receive = p_win_receive_grid(p_receive_i)

                [V, p_mot] = calcValues(p_win_if_serve, p_win_if_receive, p_cc, ...
                                        G_handle, p_aot_handle, challengesInSetRule);

                p_mot_mega{p_serve_i, p_receive_i} = p_mot;
                V_mega{p_serve_i, p_receive_i} = V;

                if (V == -999)
                    display('ERROR! Failure to converge in one of the deuces');
                    display(p_cc);
                    display(p_win_if_serve);
                    display(p_win_if_receive);
                end
            end
        end
        p_mot_mega2{challengesInSetRule-1} = p_mot_mega;
        V_mega2{challengesInSetRule-1} = V_mega;
    end
end

