// This file does stuff in Stata that were later done in Matlab (except for the predictions' confidence interval which was not calculated in Matlab.
// It requires the DB output of the calcValues.m function in Matalb to merge into the dataset (and interpolate). See the tryCalcValuesInChunks_#.m files 
// (and raw2work.do in the same folder) under the Matlab\Obsolete\ folder to see how the Matlab policy was calculated and formatted for this program to use
clear all
set more off
set logtype text
set mem 880m

cd "C:\Users\Roy\Documents\My Dropbox\Tennis\do"

cap log close
log using "..\log\p_aot_model_vs_reality.log", replace

local p_cc = 0.02
local p_cc_str = "p_cc_" + subinstr(string(`p_cc',"%3.2f"),".","p",.)

local valuePolicies_file_pref "valuesPolicies"
// Preparing Matlab's output (with values and policy functions - a value and a policy for each state in the game) to
// be merged later in the program.
foreach chals_in_set_rule in 2 3 {
	use "..\Matlab\challenge_value\\`valuePolicies_file_pref'_`chals_in_set_rule'chals_`p_cc_str'"

	gen byte chals_in_set_rule = `chals_in_set_rule'
	gen byte p_win_if_serve_xten = round(p_win_if_serve*10,1)
	gen byte p_win_if_receive_xten = round(p_win_if_receive*10,1)

	foreach pair in "0 0" "1 15" "2 30" "3 40" "4 50" {
		local sane_points : word 1 of `pair'
		local tennis_points : word 2 of `pair'
		
		foreach side in my opp {
			replace `side'points = `tennis_points' if `side'points == `sane_points' & (mygames != 6 | oppgames != 6)
		}
	}

	replace p_mot = 0 if p_mot < 0
	replace p_mot = 1 if p_mot > 1 & p_mot != .

	drop p_cc p_win_if_serve p_win_if_receive value tolerance_or_error

	/******** This part can help if some grid points in the middle are missing (but if they are, there might be a bug (we fixed it) **********
	
	// Interpolate missing grid points from adjacent grid points (for example, if (.5,.1) is missing, we assign the mean of (.7,.1) and (.3,.1))
	egen state_id = group(mysets oppsets mygames oppgames mypoints opppoints serving chals_left)
	gen probs = "_" + string(p_win_if_serve_xten) + "_" + string(p_win_if_receive_xten)
	drop p_win_if_serve_xten p_win_if_receive_xten
	reshape wide p_mot, i(state_id) j(probs) string
	forvalues srv = 1(2)9 {
		forvalues rcv = 1(2)9 { 
			if (`srv' > 1 & `srv' < 9) {
				local srv_prev = `srv' - 2
				local srv_next = `srv' + 2
				gen p_mot_interp_srv_`srv'_`rcv' = (p_mot_`srv_prev'_`rcv' + p_mot_`srv_next'_`rcv') / 2
			}
			if (`rcv' > 1 & `rcv' < 9) {
				local rcv_prev = `rcv' - 2
				local rcv_next = `rcv' + 2
				gen p_mot_interp_rcv_`srv'_`rcv' = (p_mot_`srv'_`rcv_prev' + p_mot_`srv'_`rcv_next') / 2
			}
		}
	}
	reshape long p_mot p_mot_interp_srv p_mot_interp_rcv, i(state_id) j(probs) string
	
	gen p_win_if_serve_xten   = real(regexs(1)) if regexm(probs, "_([0-9]+)_([0-9]+)")
	gen p_win_if_receive_xten = real(regexs(2)) if regexm(probs, "_([0-9]+)_([0-9]+)")
	drop probs
	
	gen p_mot_gridpoint_interpolated = (p_mot_interp_srv != . | p_mot_interp_rcv != .) if p_mot == .
	replace p_mot = p_mot_interp_srv if p_mot == .
	replace p_mot = p_mot_interp_rcv if p_mot == .
	*/
	save "..\Matlab\challenge_value\\`valuePolicies_file_pref'_`chals_in_set_rule'chals_`p_cc_str'_work", replace
}

use "..\Matlab\challenge_value\\`valuePolicies_file_pref'_2chals_`p_cc_str'_work", clear
append using "..\Matlab\challenge_value\\`valuePolicies_file_pref'_3chals_`p_cc_str'_work"


// THIS will be the file that will be merged into our data (the grid-points around our data, actually)
save "..\Matlab\challenge_value\\`valuePolicies_file_pref'_`p_cc_str'_for_merge", replace

cap erase "..\Matlab\challenge_value\\`valuePolicies_file_pref'_2chals_`p_cc_str'_work.dta"
cap erase "..\Matlab\challenge_value\\`valuePolicies_file_pref'_3chals_`p_cc_str'_work.dta"

// Start working on the (real) data, and prepare it so that we can merge the model's policy to the point.
use "..\Data\Liran edits\point_data_after_importance_round3", clear

// ONLY SINGLES MEN GAMES
keep if !doubles & !female

rename sets_won_chal mysets
rename sets_won_opp  oppsets
rename curset_chal   mygames
rename curset_opp    oppgames
rename game_chal     mypoints
rename game_opp      opppoints
rename chal_serving  serving
rename challenges_left chals_left

// Getting the four grid points around this player's probability to win point if serving/receiving.
//     1|     |2
//   ---+-----+----
//      |  .  |
//     3|     |4 
//   ---+-----+----
//      |     |
// So for the given dot's (p_win_serve, p_win_receive), or (x-axis, y-axis) we get four pairs:
// 1. (floor(p_win_serve), ceil(p_win_receive))
// 2. (ceil(p_win_serve) , ceil(p_win_receive))
// 3. (floor(p_win_serve), floor(p_win_receive))
// 4. (ceil(p_win_serve) , floor(p_win_receive))
//
// One last thing, we need to multiply by 10 the probabilities, to avoid Stata's problems with non-integers
// and rounding them to merge the other dataset later. We also make those p_win_serve and p_win_receive
// to be bounded by 0.1 from below and 0.9 from above (only two observations fail to be in this range in the first place)

gen mtch_chal_pt_win_prob_srv_b = max(min(mtch_chal_pt_win_prob_srv, .8999),.1) if mtch_chal_pt_win_prob_srv != .
gen mtch_chal_pt_win_prob_rcv_b = max(min(mtch_chal_pt_win_prob_rcv, .8999),.1) if mtch_chal_pt_win_prob_srv != .

gen byte floor_serve   = (mtch_chal_pt_win_prob_srv_b - mod(mtch_chal_pt_win_prob_srv_b-.1,.2))*10
gen byte floor_receive = (mtch_chal_pt_win_prob_rcv_b - mod(mtch_chal_pt_win_prob_rcv_b-.1,.2))*10

gen byte p_win_if_serve_xten1 	= floor_serve
gen byte p_win_if_receive_xten1 = floor_receive + 2

gen byte p_win_if_serve_xten2 	= floor_serve   + 2
gen byte p_win_if_receive_xten2 = floor_receive + 2

gen byte p_win_if_serve_xten3 	= floor_serve
gen byte p_win_if_receive_xten3 = floor_receive

gen byte p_win_if_serve_xten4 	= floor_serve   + 2
gen byte p_win_if_receive_xten4 = floor_receive

gen challenge_id = _n

// Translate tiebreak scores above 6:6 to 6:6
gen orig_mypoints  = mypoints
gen orig_opppoints = opppoints

replace mypoints  = 6 + (orig_mypoints == orig_opppoints + 1) if tiebreak & orig_mypoints > 6 & orig_opppoints > 6 & orig_mypoints != .
replace opppoints = 6 + (orig_opppoints == orig_mypoints + 1) if tiebreak & orig_mypoints > 6 & orig_opppoints > 6 & orig_mypoints != .

// Most tournaments in our data had 2 challenges in the bank in the beginning of every set.
// However, tournaments 26 and 27 had 3 challenges in the bank in the beginning of every set.
gen chals_in_set_rule = 2 + (tournament_id == 26 | tournament_id == 27)

// Make an observation of every grid point.
reshape long p_win_if_serve_xten p_win_if_receive_xten, i(challenge_id) j(gridpoint)

// Merge the p_mot's from the Matlab's grid of value and policy calculation
merge m:1 mysets oppsets mygames oppgames mypoints opppoints serving chals_left ///
		p_win_if_serve_xten p_win_if_receive_xten chals_in_set_rule ///
				using "..\Matlab\challenge_value\\`valuePolicies_file_pref'_`p_cc_str'_for_merge", keep(master match)

tab _merge serving, m

count if _merge == 3 & p_mot == .

// Apply the bilinear interpolation formula (not in the same order of the gridpoints above):
// f(x,y) = (1/.2^2) * [	f(x1,y1) * (x2-x)*(y2-y) +        (gridpoint 3)
//							f(x2,y1) * (x-x1)*(y2-y) +        (gridpoint 4)
//							f(x1,y2) * (x2-x)*(y-y1) +        (gridpoint 1)
//							f(x2,y2) * (x-x1)*(y-y1) ]        (gridpoint 2)
//
// where x# and y# are the coordinates of the relevant gridpoints and f(x#,y#) is the p_mot of the gridpoint, 
// and x and y are the exact arguments for which we are trying to estimate (interpolate) the value of the function.
// We'll do it by calculating each row for each observation (each gridpoint) and then sum it up.

gen 	contribution = (1/.2^2) * p_mot * (floor_serve/10 + .2 - mtch_chal_pt_win_prob_srv_b) * (floor_receive/10 + .2 - mtch_chal_pt_win_prob_rcv_b) if gridpoint == 3
replace contribution = (1/.2^2) * p_mot * (mtch_chal_pt_win_prob_srv_b - floor_serve/10     ) * (floor_receive/10 + .2 - mtch_chal_pt_win_prob_rcv_b) if gridpoint == 4
replace contribution = (1/.2^2) * p_mot * (floor_serve/10 + .2 - mtch_chal_pt_win_prob_srv_b) * (mtch_chal_pt_win_prob_rcv_b - floor_receive/10)      if gridpoint == 1
replace contribution = (1/.2^2) * p_mot * (mtch_chal_pt_win_prob_srv_b - floor_serve/10     ) * (mtch_chal_pt_win_prob_rcv_b - floor_receive/10)      if gridpoint == 2

egen p_mot_interp = sum(contribution), by(challenge_id)

gen miscontrib = contribution == .
egen miscontrib_in_point = max(miscontrib), by(challenge_id)

// egen sum() turns missing values to zeros.
replace p_mot_interp = . if miscontrib_in_point

// Insist on those with some nonmissing gridpoints. Specifically, those that miss points for p_win_if_serve = 0.9
// and nonmissings for p_win_if_serve = 0.7, with the true p_win_if_serve in (0.7,0.8)... those with p_win_if_serve>0.8 
// will be harder to justify interpolating only from p_win_if_serve = 0.7.
gen 	linear_contrib = p_mot * (mtch_chal_pt_win_prob_rcv_b - floor_receive/10)		/ .2 if gridpoint == 1 & p_win_if_serve_xten == 7
replace linear_contrib = p_mot * (floor_receive/10 + .2 - mtch_chal_pt_win_prob_rcv_b) 	/ .2 if gridpoint == 3 & p_win_if_serve_xten == 7
egen linear_thing = sum(linear_contrib), by(challenge_id)

gen miscontrib2 = linear_contrib == .
egen miscontrib_in_point2 = sum(miscontrib2), by(challenge_id)

replace p_mot_interp = linear_thing if miscontrib_in_point & miscontrib_in_point2 == 2 & mtch_chal_pt_win_prob_srv < .8
drop miscontrib* linear_contrib linear_thing

keep if gridpoint == 3

replace mypoints = orig_mypoints
replace opppoints = orig_opppoints

drop p_mot gridpoint contribution orig_mypoints orig_opppoints

count if _merge == 3 & p_mot_interp == .

// OK, we have p_mot interpolations. We can get p_aot according to the model:
gen p_aot_model = .295 + .705*p_mot_interp


// REGRESSING challenge_won on the model's implied mean winning. If the model fits perfectly we should get a constant of 0 and a slope of 1/
reg challenge_won p_aot_model
test (_cons=0) (p_aot_model=1)

su p_aot_model
local minrange = r(min)
local maxrange = r(max)

twoway (lfitci challenge_won p_aot_model, clw(thick) fcolor(blue) fintensity(5) alp(dot)) (function y=x, range(`minrange' `maxrange') lc(red)) , legend(order (2 3) lab(2 "Proportion won in data") lab(3 "Model (45-deg line)"))  ///
		title("Comparing Model Implied and Actual Probability to Win Challenge") xtitle("Pr[win|state] according to the model") ytitle("Pr[win]")
graph export "..\out\model_vs_reality.png", replace

reg challenge_won p_mot_interp

// Constraining the line to pass through (1,1)
// y = alpha + beta*x   AND  1 = alpha + beta * 1  => alpha = 1 - beta
//    =>  y = 1 - beta + beta*x
//    =>  y-1 = beta*(x-1)
gen y_1 = challenge_won - 1
gen x_1 = p_mot_interp - 1

reg y_1 x_1, nocons
di "Constrained beta = " round(_b[x_1], 0.001)
local alph = 1 - _b[x_1]
di "Constrained alpha = " round(`alph', 0.001)

su p_mot_interp
local minrange = r(min)
local maxrange = r(max)

twoway (lfitci challenge_won p_mot_interp, clw(thick) fcolor(blue) fintensity(5) alp(dot)) (function y=.295 + .705*x, range(`minrange' `maxrange') lc(red)) ///
		(function y=x, range(`minrange' `maxrange') lc(black)) , legend(order (2 3 4) lab(2 "Proportion won in data") lab(3 "Model (0.295 + 0.705*x)") lab(4 "45-degrees line"))  ///
		title("Comparing Model Implied and Actual Probability to Win Challenge") subtitle("As functions of the policy P_mot") xtitle("P_mot (policy)") ytitle("Pr[win] by model and data-prediction")
graph export "..\out\model_vs_reality_p_mot.png", replace

/* exporting to text, as Liran asked ...
gen temp_x = (_n-1)/100*(`maxrange'-`minrange') + `minrange' in 1/100
reg challenge_won p_mot_interp
rename p_mot_interp p_mot_interp_orig
rename temp_x p_mot_interp
predict std, stdp
predict gph_line, xb
gen ci_ub = gph_line + 1.96*std
gen ci_lb = gph_line - 1.96*std
gen p_aot_model = .295 + .705*p_mot_interp
*/

lpoly challenge_won p_mot_interp if p_mot_interp < .18, bwidth(0.025) ci generate(nonparam_pmot nonparam_paot) ///
		title("Actual Probability to Win Challenge, nonparametrically") subtitle("As functions of the policy P_mot") xtitle("P_mot (policy)") ytitle(`""Local" Pr[win]"')
graph export "..\out\model_vs_reality_p_mot_nonparametric.png", replace

twoway (line nonparam_paot nonparam_pmot) (function y = .295+ .705*x) (function y = x) , ///
	title("Comparing Model- and Actual-Probability to Win Challenge") subtitle("Nonparametrically, as functions of the policy P_mot") xtitle("P_mot (policy)") ytitle("Pr[win] by model and data-prediction") ///
	legend(lab(1 "Kernel-estimated proportion won in data") lab(2 "Model (0.295 + 0.705*x)") lab(3 "45-degrees line"))
graph export "..\out\model_vs_reality_p_mot_nonparametric_superimposed.png", replace

/*
// For the convex version of P_aot(P_mot) with alpha=2 in the Beta distribution of p_mot...
gen p_mot_interp_sq = p_mot_interp ^ 2
nl (challenge_won = (2/(2+{beta}))*(({beta}/2)*(1+{beta})*p_mot_interp_sq/(1+{beta}*p_mot_interp)+1)), ///
		variables( p_mot_interp p_mot_interp_sq )

local b = _b[beta:_cons]
local b_theory = 2*(1/.295-1)

twoway (function y =(2/(2+`b_theory'))*((`b_theory'/2)*(1+`b_theory')*x^2/(1+`b_theory'*x)+1)) ///
		 (function y =(2/(2+`b'))*((`b'/2)*(1+`b')*x^2/(1+`b'*x)+1)) ///
		 (function y=x), ///
			title("Comparing Model Implied and Actual Probability to Win Challenge") subtitle("Assuming a convex G function, specifically q~Beta(2,\beta)") ///
			legend(lab(1 "beta=`b_theory' as assumed in model") lab(2 "beta=`b' as estimated from data") lab(3 "45-degrees line"))
graph export "..\out\model_vs_reality_p_mot_convex_G_assumed.png", replace

twoway (line nonparam_paot nonparam_pmot)  (function y =(2/(2+`b_theory'))*((`b_theory'/2)*(1+`b_theory')*x^2/(1+`b_theory'*x)+1)) (function y = x), ///
	title("Comparing Model- and Actual-Probability to Win Challenge") subtitle("Nonparametrically, as functions of the policy P_mot") xtitle("P_mot (policy)") ytitle("Pr[win] by model and data-prediction") ///
	legend(lab(1 "Kernel-estimated proportion won in data") lab(2 "Model (0.295 + 0.705*x)") lab(3 "45-degrees line"))
graph export "..\out\model_vs_reality_p_mot_nonparametric_superimposed_convex_G_assumed.png", replace

// Short thing for the table (to show how many obs between grid points)
table floor_serve floor_receive , c(N p_mot_interp)

*/






/* STUFF FROM EARLIER TRIALS

// -=-=-=-=-=- Analyze by pairs of "scenarios" -=-=-=-=-=-

cap erase "..\out\p_aot_responsiveness.txt"
cap erase "..\out\p_aot_responsiveness.xml"

// Compare situations and challenges' results
// 		2.1 Game score 40-40 vs. 40-0
gen 	sit2_1 = 1 if mypoints == 40 & opppoints == 40
replace sit2_1 = 0 if mypoints == 40 & opppoints == 0
label variable sit2_1 "Game score 40-40 vs. 40-0"

// Compare situations and challenges' results
// 		2.1b Game score 40-40 vs. 40-0 or 0-40
gen 	sit2_1b = 1 if mypoints == 40 & opppoints == 40
replace sit2_1b = 0 if (mypoints == 40 & opppoints == 0) |  ///
						(mypoints == 0 & opppoints == 40)
label variable sit2_1b "Game score 40-40 vs. 40-0 or 0-40"

//		2.2 Break point vs. other points
gen 	sit2_2 = 1 if serving == 1 & (opppoints == 40 & mypoints < 40 | opppoints == 50 & mypoints < 50)
replace sit2_2 = 1 if serving == 0 & (mypoints == 40 & opppoints < 40 | mypoints == 50 & opppoints < 50)
replace sit2_2 = 0 if serving != . & sit2_2 == .
label variable sit2_2 "Break point (chal or opp) vs. other points"

//		2.3 High pwm_if_win vs. low pwm_if_win
su pwm_if_won, detail
local highthres = r(p75)
local lowthres = r(p25)
gen 	sit2_3 = 1 if pwm_if_won > `highthres' & pwm_if_won != .
replace	sit2_3 = 0 if pwm_if_won < `lowthres'
label variable sit2_3 "High PWM if pt won (> 75 pctile) vs. Low PWM if pt won (< 25 pctile)"

gen pwm_diff = pwm_if_won - pwm_if_lst

//		2.4 High (pwm_if_win-pwm_if_lst) vs. low (pwm_if_win-pwm_if_lst)
su pwm_diff, detail
local highthres = r(p75)
local lowthres = r(p25)
gen 	sit2_4 = 1 if pwm_diff > `highthres' & pwm_diff != .
replace	sit2_4 = 0 if pwm_diff < `lowthres'
label variable sit2_4 "High PWM if won-lost diff (> 75 pctile) vs. Low PWM diff (< 25 pctile)"

//		2.5 Challenges left 1 vs. challenges left more than 1
gen		sit2_5 = chals_left > 1
label variable sit2_5 "Challenges left (before this one) > 1 vs. Challenges left == 1"

//		2.6 Top 20 players vs. not Top 20 player
gen		sit2_6 =  challenge_maker_rank1 <= 20
label variable sit2_6 "Challenger ranking top 20 vs. Challenger ranking lower than top 20"

// 		2.7 Tiebreak vs. not a tiebreak
gen		sit2_7 = tiebreak
label variable sit2_7 "Tiebreak vs. not a tiebreak"

// 		2.8 Tiebreak vs. not a tiebreak
gen		sit2_8 = round <= 8
label variable sit2_8 "1/4 final to final vs. lower rounds than 1/4 final"

// Now compare those situations, post and plot
// NEW TABLE (consistent id 8): COMPARING P_AOT RESPONSIVENESS IN BINARY CASES
cap postclose sitComparisons
postfile sitComparisons str80 sitDesc str32 depVar N1 N0 mean1 mean0 diff diff_se med1 med0 ///
		pr_same_med pr_same_dist_mww pr_same_dist_ks using "..\out\p_aot_comparisons", replace

su p_aot_model
local minrange = r(min)
local maxrange = r(max)

foreach situation in 2_1 2_1b 2_2 2_3 2_4 2_5 2_6 2_7 2_8 {
	local label : variable label sit`situation'
	
	cap drop former
	gen former = sit`situation'
	
	reg challenge_won former, robust
	outreg2 using "..\out\p_aot_responsiveness.xml", excel auto(6)
	
	reg challenge_won former p_aot_model, robust
	outreg2 using "..\out\p_aot_responsiveness.xml", excel auto(6)
	
	cap drop formerXPaot
	gen formerXPaot = former * p_aot_model

	reg challenge_won former p_aot_model formerXPaot, robust
	outreg2 using "..\out\p_aot_responsiveness.xml", excel auto(6)

	qui su p_aot_model if former != .
	local minrange = r(min)
	local maxrange = r(max)

	local formertext = substr("`label'", 1, strpos("`label'", "vs.")-2)
	local lattertext = substr("`label'", strpos("`label'", "vs.")+3, strlen("`label'")-strpos("`label'", "vs.")-2)
	
	twoway 	(lfitci challenge_won p_aot_model if former==1, clw(thick) fcol(none) alc(blue) clc(blue) alp(shortdash)) ///
			(lfitci challenge_won p_aot_model if former==0, clw(thick) fcol(none) alc(green) clc(green) alp(shortdash)) ///
			(function y=x, range(`minrange' `maxrange') lc(red)) , ///
		 legend(order(2 4 5) lab(2 "Prop won (`formertext')") lab(4 "Prop won (`lattertext')") lab(5 "Model (45-deg line)"))  ///
		title("Comparing Model Implied and Actual Probability to Win Challenge") xtitle("Pr[win|state] according to the model") ytitle("Pr[win]")
	graph export "..\out\model_vs_reality_sit`situation'.png", replace
}



// -=-=-=-=-=- Analyze by bins of p_mot assigned by the model  -=-=-=-=-=-


// Assigning each challenge its bin according to the following percentiles as separators:
_pctile p_mot_interp, percentiles(40 50 60 70 80 90 95)

gen cutoff = .
forvalues cutoff_ind = 1/7 {
	replace cutoff = r(r`cutoff_ind') in `cutoff_ind'
}

xtile p_mot_bin = p_mot_interp , cut(cutoff)

su p_mot_bin
local tot_obs = r(N)

postfile table bin p_mot_from p_mot_to N_obs prop_obs p_aot_model_mean prop_really_won ///
	using "..\out\model_vs_real", replace
	
forvalues bin = 1/8 {
	su p_mot_interp if p_mot_bin == `bin'
	local N_obs      = r(N)
	local prop_obs	 = `N_obs' / `tot_obs'
	local p_mot_from = r(min)
	local p_mot_to   = r(max)
	su p_aot_model  if p_mot_bin == `bin'
	local p_aot_model_mean = r(mean)
	su challenge_won  if p_mot_bin == `bin'
	local prop_won = r(mean)
	post table (`bin') (`p_mot_from') (`p_mot_to') (`N_obs') (`prop_obs') (`p_aot_model_mean') (`prop_won')
}

su p_aot_model
local p_aot_model_mean = r(mean)
su challenge_won 
local prop_won = r(mean)

post table (.) (.) (.) (`tot_obs') (1) (`p_aot_model_mean') (`prop_won')

postclose table

preserve

use "..\out\model_vs_real", clear
outsheet using "..\out\model_vs_real.csv", replace comma

restore

xi: reg challenge_won i.p_mot_bin, robust

*/
log close
