/* THIS IS THE MAIN DATA-PREPARATION PROGRAM
This program does the following:

1. It takes the challenges data
2. It merges the match-level data to it (to fill in holes in scores and to get probabilities to win a point)
3. It identifies server-related challenges (according to the lines) and defines who served according to the call
   (servers challenge calls out and receivers calls in).
4. It populates (imputes) the servers of the other challenges in same match (as the serve-related challenge)
   according to the score and whether it's a tiebreaker
5. It calculates the probabilities to win a serving point and a receiving point.
6. It calls the program that calculates probability to win the whole match (1) if challenge point is won
   (2) if challenge point is lost (3) if challenge point is replayed.
*/


set more off
clear all

// SET DATA FOLDER HERE.
cd "C:\Users\Roy\Documents\My Dropbox\Tennis\Data\Liran edits\"

use match_data
append using shanghai_for_roy

egen max_chal = rowmax(challenger?_id)
egen max_opp  = rowmax(opponent?_id)
egen max_player = rowmax(max_chal max_opp)
egen min_player = rowmin(max_chal max_opp)

replace round = 999 if round == .

sort tournament_id max_player min_player round 
egen tag_dup = tag(tournament_id max_player min_player round)

// Drop duplicates
drop if tag_dup == 0

rename opp_stats_service_games  opp_stats_service_games_played

// Renaming variables to be later used after merge.
foreach side in chal opp {
	forvalues set = 1/3 {
		rename score`set'_`side' `side'_score_`set'
	}
}

foreach side in chal opp {
	foreach var of varlist `side'_stats_* {
		local base_name = substr("`var'", 8+strlen("`side'"), strlen("`var'")-7-strlen("`side'"))
		rename `side'_stats_`base_name' mtch_`side'_`base_name'
	}
}

foreach var of varlist chal_* opp_* {
	rename `var' mtch_`var'
}

rename max_chal mtch_max_chal
rename max_opp  mtch_max_opp

/*	local base_name = substr("`var'", 6, strlen("`var'")-5)
	gen		minp_`base_name' = `var'
	replace	minp_`base_name' = opp_`base_name' if min_player == max_opp
	
	gen		maxp_`base_name' = `var'
	replace maxp_`base_name' = opp_`base_name' if max_player == max_opp
}*/

//drop chal_* opp_*

save match_data_for_merge, replace


use point_data7
// cd "C:\Documents and Settings\RM\My Documents\My Dropbox\Ran\Data\Liran edits\"

egen max_chal = rowmax(challenger?_id)
egen max_opp  = rowmax(opponent?_id)
egen max_player = rowmax(max_chal max_opp)
egen min_player = rowmin(max_chal max_opp)

replace round = 999 if round == .

joinby tournament_id max_player min_player round using match_data_for_merge, unmatched(both) update

tab _merge

// First, make sure the arbitrary challengers and opponents-dependent variables from 
// the match dataset will match the challenger and opponent side of each challenge (they
// change here in the challenge-based dataset)

// (before that, rename rank variables to have names like mtch_chal_... and mtch_opp_...)
rename challenge_maker_rank1 mtch_chal_rank1
rename challenge_maker_rank2 mtch_chal_rank2
rename opponent_rank1 mtch_opp_rank1
rename opponent_rank2 mtch_opp_rank2

gen temp = .
foreach var of varlist mtch_chal_* {
	local base_name = substr("`var'", 11, strlen("`var'")-10)
	replace temp 					= mtch_opp_`base_name'
	replace mtch_opp_`base_name' 	= mtch_chal_`base_name' if mtch_max_chal != max_chal
	replace mtch_chal_`base_name' 	= temp 					if mtch_max_chal != max_chal
}

drop temp _merge

// (rename back (regressions already depend on the old name))
rename mtch_chal_rank1 challenge_maker_rank1 
rename mtch_chal_rank2 challenge_maker_rank2 
rename mtch_opp_rank1 opponent_rank1 
rename mtch_opp_rank2 opponent_rank2 

// Set score (game-game) problems
// Take previous sets scores from the match-data
gen missing_1st_set_chal = (set1_chal == . & (set2_chal != . | set3_chal != .))
gen missing_1st_set_opp  = (set1_opp  == . & (set2_opp  != . | set3_opp  != .))
gen missing_2nd_set_chal = (set2_chal == . & set3_chal != .)
gen missing_2nd_set_opp  = (set2_chal == . & set3_chal != .)

replace set1_chal = mtch_chal_score_1 if (set1_chal == . & (set2_chal != . | set3_chal != .))
replace set1_opp  = mtch_opp_score_1  if (set1_opp  == . & (set2_opp  != . | set3_opp  != .))

replace set2_chal = mtch_chal_score_2 if (set2_chal == . & set3_chal != .)
replace set2_opp  = mtch_opp_score_2  if (set2_opp  == . & set3_opp  != .)

// But since the umpire don't write 0:0 in games in the beginning of the set some times
// we need to see if a second set has the first set's result and the first is missing. 
// If this is the case we need to reset the 2nd set to 0:0
gen missing_1st_in_2nd = missing_1st_set_chal & missing_1st_set_opp & set1_chal == set2_chal & set1_opp == set2_opp
replace set2_chal = 0 if missing_1st_in_2nd
replace set2_opp  = 0 if missing_1st_in_2nd

// missing scores for first set and match-time which is really low mean that
// it was 0-0 in sets
replace set1_chal = 0 if set == 1 & set1_chal == . & set1_opp == . & match_time < 10
replace set1_opp  = 0 if set == 1 & set1_chal == 0 & set1_opp == . & match_time < 10

// missing scores for first set in 0-0 and really at the beginning of the game (first 3 minutes)
// mean that it was 0-0 in game
replace game_chal = 0 if set == 1 & set1_chal == 0 & set1_opp == 0 & game_chal == . & game_opp == . ///
							& match_time < 3
replace game_opp  = 0 if set == 1 & set1_chal == 0 & set1_opp == 0 & game_chal == . & game_opp == . ///
							& match_time < 3

gen tiebreak = 0
replace tiebreak = 1 if (set==1 & set1_chal==6 & set1_opp==6)
replace tiebreak = 1 if (set==2 & set2_chal==6 & set2_opp==6)
replace tiebreak = 1 if (set==3 & set3_chal==6 & set3_opp==6)

// For doubles, the third set is the 10-points tiebreaker. This might have been coded into
// the third set instead of the game points. If not, then the third set will have missing
// values too, so we didn't change anything
gen doubles_prob = (game_chal == . & game_opp == . & doubles == 1 & set == 3)
replace game_chal = set3_chal if doubles_prob
replace game_opp  = set3_opp  if doubles_prob
replace set3_chal = 0 if doubles_prob
replace set3_opp = 0  if doubles_prob
replace tiebreak = 1 if doubles_prob

drop doubles_prob

// Incrementing set if current set's score indicates that it should be over.
foreach side in chal opp {
	gen cur_set_`side' = .
	forvalues s = 1/3 {				
		replace cur_set_`side' = set`s'_`side' if set == `s'
	}
}

gen cur_set_sum_games = cur_set_opp + cur_set_chal
gen set_should_be_over = (cur_set_chal >= 6 | cur_set_opp >= 6) & (cur_set_chal != 6 | cur_set_opp != 6) & cur_set_sum_games != 11 ///
							if cur_set_sum_games != .
tab set_should_be_over, m
list doubles set-game_opp if set_should_be_over == 1

// If the set should be over and the next set has missing values for the games won in set, it is
// probably the umpire forgetting to write the 0-0. Make it a 0-0
forvalues s=2/3 {
	replace set`s'_chal = 0 if set`s'_chal == . & set`s'_opp == . & set+1 == `s' & set_should_be_over
	replace set`s'_opp  = 0 if set`s'_chal == 0 & set`s'_opp == . & set+1 == `s' & set_should_be_over
}
replace set = set + 1 if set_should_be_over == 1

list doubles set-game_opp if set_should_be_over == 1

drop cur_set_chal cur_set_opp
// Incrementing set if current set's score indicates that it should be over.
foreach side in chal opp {
	gen cur_set_`side' = .
	forvalues s = 1/3 {				
		replace cur_set_`side' = set`s'_`side' if set == `s'
	}
}


gsort tournament_id max_player min_player round, gen(match_id)
drop max_opp max_player min_player

gsort match_id match_time

// I ASSUME THAT MISSING VALUES IN GAME_CHAL ARE ZEROES. CHECK THAT.
egen gamepoints_sum = rowtotal(game_chal game_opp)

egen cumulative_games = rowtotal(set?_chal set?_opp)


///////////// DETERMINE WHO IS SERVING //////////////
gen chal_served_point = (line=="LC" | line =="RC" | line=="LS" | line =="RS") * (~call_in) if (line=="LC" | line =="RC" | line=="LS" | line =="RS")

// If it's a tie-breaker then we want the first server of this tie breaker
// Servers go 1;0;0;1;1;0;0;1;1;0;0... in a tie-breaker. Then if it's a tiebreaker
// and it's the first two points of every four point as of the second point (0;0; of the 0;0;1;1;)
// the challenger started the game on the other side of the serve.
gen chal_served_game = chal_served_point
replace chal_served_game = 1 - chal_served_point if tiebreak == 1 & gamepoints_sum > 0 ///
																& mod(gamepoints_sum-1, 4) <= 1

// Take the first challenge that is a serve and make it the source for
// populating serves to all other challenges (using obs number instead of match time because
// some games have more than one challenge with the same match_time).
// source_obs should be 1 for the source, 0 for other challenges in games that have
// at least one serve-challenge and missing value for games with no serve-challenge.
gen obs_num_if_serve = _n if chal_served_game != .
egen min_obs_num = min(obs_num_if_serve), by(match_id)
gen source_obs = _n == min_obs_num if min_obs_num != . 
drop obs_num_if_serve min_obs_num

// We want to populate whether source served or received to all other obs
gen source_served_game_tmp = source_obs * chal_served_game
egen source_served_game = max(source_served_game_tmp), by(match_id)
drop source_served_game_tmp

// We need to know for each observation of the game (for which we have at least one
// serve-challenge) if it was the same challenging side, and if it had the same parity
// (even/odd) of the game, as the Source challenge.
gen game_is_odd = mod(cumulative_games, 2)
gen source_is_odd_tmp = game_is_odd if source_obs == 1
egen source_is_odd = max(source_is_odd_tmp), by(match_id)
drop source_is_odd_tmp

gen source_challenger_tmp = max_chal if source_obs == 1
egen source_challenger = max(source_challenger_tmp), by(match_id)
drop source_challenger_tmp

// We have the source challenger's serving status (1 if served, 0 if received)
// and we want to flip this status for this observation if (1) it's not the same
// challenger AND (2) it's not the same game parity as the source.
// (we can flip the status twice). A flip is like a XOR operation of the original
// status (x) and the flip flag (y). In other words: x + y mod 2.
// original		flip	result
// 1			0		1
// 1			1		0
// 0			0		0
// 0			1		1
gen chal_served_game_populated = mod(mod(source_served_game + ///
										(game_is_odd != source_is_odd), 2) + ///
									 (source_challenger != max_chal), 2)   // remember, max_chal is the id of the point's challenger

// Point was served if game was served, unless game was a tiebreaker and then
// we should flip if  it's the first two points of every four point as of the 
// second point (0;0; of the 0;0;1;1;) the challenger started the game on the 
// other side of the serve.
gen 	chal_served_point_populated = chal_served_game_populated
replace chal_served_point_populated = 1 - chal_served_game_populated if tiebreak == 1 & gamepoints_sum > 0 ///
																& mod(gamepoints_sum-1, 4) <= 1

// Check conflicts (off diagonal elements)
tab chal_served_point chal_served_point_populated

// SECOND TIMEEEEE - For matches with more conflicts than good matches, change source_obs so that the 
// conflicts will be the source
gen server_conf = chal_served_point != chal_served_point_populated if chal_served_point != .
gen server_right = chal_served_point == chal_served_point_populated if chal_served_point != .
egen confs_in_match = sum(server_conf), by(match_id)
egen rights_in_match = sum(server_right), by(match_id)

gen source_was_bad = confs_in_match > rights_in_match if confs_in_match > 0 & confs_in_match != .

// Assign new source (for which more observations are aligned with, and REDO Calculation)
gen good_source_obs = _n if server_conf == 1 // what was conflict before will now be the source,
                                             // so it will make the others opposite
egen min_good_source_obs = min(good_source_obs), by(match_id)
replace source_obs = _n == min_good_source_obs if source_was_bad == 1
drop good_source_obs min_good_source_obs

	// We want to populate whether source served or received to all other obs
	gen source_served_game_tmp = source_obs * chal_served_game
	egen source_served_game_2nd = max(source_served_game_tmp), by(match_id)
	replace source_served_game = source_served_game_2nd if source_was_bad == 1
	drop source_served_game_tmp source_served_game_2nd
	
	// We need to know for each observation of the game (for which we have at least one
	// serve-challenge) if it was the same challenging side, and if it had the same parity
	// (even/odd) of the game, as the Source challenge.
	gen source_is_odd_tmp = game_is_odd if source_obs == 1
	egen source_is_odd_2nd = max(source_is_odd_tmp), by(match_id)
	replace source_is_odd = source_is_odd_2nd if source_was_bad == 1
	drop source_is_odd_tmp source_is_odd_2nd
	
	gen source_challenger_tmp = max_chal if source_obs == 1
	egen source_challenger_2nd = max(source_challenger_tmp), by(match_id)
	replace source_challenger = source_challenger_2nd if source_was_bad == 1
	drop source_challenger_tmp source_challenger_2nd
	
	replace chal_served_game_populated = mod(mod(source_served_game + ///
											(game_is_odd != source_is_odd), 2) + ///
										 (source_challenger != max_chal), 2) ///
								if source_was_bad == 1
	
	// Point was served if game was served, unless game was a tiebreaker and then
	// we should flip if  it's the first two points of every four point as of the 
	// second point (0;0; of the 0;0;1;1;) the challenger started the game on the 
	// other side of the serve.
	replace	chal_served_point_populated = chal_served_game_populated ///
									if source_was_bad == 1
	replace chal_served_point_populated = 1 - chal_served_game_populated ///
									if source_was_bad == 1 & (tiebreak == 1 & gamepoints_sum > 0 ///
																	& mod(gamepoints_sum-1, 4) <= 1)

// CHECK conflicts again
tab chal_served_point chal_served_point_populated

// Check how many serve information we imputed (before vs after)
tab1 chal_served_point chal_served_point_populated, missing


// IMPUTATIONS OF GAME POINTS
// 1. Translating "Adv" to 50-40
replace game_chal = 40 if game_chal == . & game_opp  == 50
replace game_opp  = 40 if game_opp  == . & game_chal == 50

// 3. Imputing 0-0 for .-. unless .-. is the game points for all challenges in match
//    (in which case it's probably the Umpire's negligence)
gen mis_game_pts = (game_chal == . 	& game_opp == .)
egen mis_game_pts_in_match = sum(mis_game_pts), by(match_id)
egen tot_chal_in_match = count(mis_game_pts), by(match_id)
gen mis_game_pts_forall = mis_game_pts_in_match == tot_chal_in_match

tab mis_game_pts mis_game_pts_forall, m

gen game_pts_imputed = game_chal == . & game_opp == . & !mis_game_pts_forall
replace game_chal = 0 if game_pts_imputed
replace game_opp  = 0 if game_pts_imputed

//// Different checks for data validity
// Check missings
gen 	miscode = 0 if 				  cur_set_chal == . & cur_set_opp == .
replace miscode = 1 if miscode == . & (cur_set_chal == . | cur_set_opp == .)
replace miscode = 2 if miscode == . & (game_chal == . 	& game_opp == .)
replace miscode = 3 if miscode == . & (game_chal == . 	| game_opp == .)
replace miscode = 99 if miscode == .

label define miscode 0 "Games won in set missing for both" 1 "Games won in set missing for one side only" ///
					2 "Game pts miss for both" 3 "Game pts miss for one,the other!=50" ///
					99 "No missing problem"
					
label values miscode miscode

tab miscode, miss

// Check that game points make sense.
gen game_score_prob = ((game_chal != 0 & game_chal != 15 & game_chal != 30 & game_chal != 40) | ///
					  (game_opp  != 0 & game_opp  != 15 & game_opp  != 30 & game_opp  != 40)) & gamepoints_sum != 90 ///
					  if !tiebreak & (game_chal != . & game_opp != .)

replace game_score_prob = 1 if tiebreak & ///
			((((game_chal >= 7 | game_opp >= 7) & !doubles) | ((game_chal >= 10 | game_opp >= 10) & doubles)) & ///
			(abs(game_chal - game_opp) >= 2)) & (game_chal != . & game_opp != .)

tab game_score_prob, m

// flagging matches in which populating the server contradicts another serve-related challenge
gen contradiction = chal_served_point != chal_served_point_populated & chal_served_point != .
egen contrad_in_match = max(contradiction), by(match_id)

gen mtch_chal_pt_win_prob_srv = mtch_chal_service_points_won1 / mtch_chal_service_points_won2
gen mtch_chal_pt_win_prob_rcv = mtch_chal_return_points_won1 / mtch_chal_return_points_won2


// SERVING RULE - if you have server directly from this challenge, take it. Otherwise, 
// 				impute it from other challenges in match
gen 	chal_serving = chal_served_point
replace chal_serving = chal_served_point_populated if chal_serving == .

gen curset_chal = .
gen curset_opp  = .

forvalues s = 1/3 {
	gen set`s'_won  = set`s'_chal - 2 >= set`s'_opp  & set`s'_chal >= 6 & set`s'_chal != .
	gen set`s'_lost = set`s'_opp  - 2 >= set`s'_chal & set`s'_opp  >= 6 & set`s'_opp  != .
	
	replace curset_chal = set`s'_chal if set == `s'
	replace curset_opp  = set`s'_opp  if set == `s'
}

egen sets_won_chal = rowtotal(set?_won)
egen sets_won_opp = rowtotal(set?_lost)
drop set1_won-set3_lost

gen scoreprob = sets_won_chal == 2 | sets_won_opp == 2 | ///
				(!tiebreak & game_chal != 0 & game_chal != 15 & game_chal != 30 & game_chal != 40 & game_chal != 50) | ///
				(!tiebreak & game_opp != 0 	& game_opp != 15  & game_opp != 30  & game_opp != 40  & game_opp != 50) | ///
				curset_chal == . | curset_opp == . | sets_won_chal == . | sets_won_opp == .


/////////// END OF DATA PREPARATION /////////
save point_data_before_importance, replace

/* THIS PART WAS USED WHEN WE MERGED BACK THE SHANGHAI MATCHES. NO NEED IN THIS IF YOU RUN THE
   PROGRAM FROM THE BEGINNING (WHEN YOU WANT TO CALCULATE PWM FOR ALL CHALLENGES
replace round = . if round == 999
joinby tournament_id  challenger1_id challenger2_id round opponent1_id opponent2_id match_time set  ///
		call_in hawk_in challenge_won distance_mm doubles female line ///
		using "point_data_after_importance.dta", unmatched(both) update

replace round = 999 if round == .
egen missing_pwm = rowmiss(pwm_*)

*/

// PWM is Probability to Win Match
gen pwm_if_won = .
gen pwm_if_lst = .
gen pwm_if_rpl = .
gen epl_if_won = .
gen epl_if_lst = .
gen epl_if_rpl = .
gen error_in_pwm = .

label variable pwm_if_won "Probability to win match if challenge won"
label variable pwm_if_lst "Probability to win match if challenge lost"
label variable pwm_if_rpl "Probability to win match if challenge replayed"
label variable epl_if_won "Expected points left in set if challenge won"
label variable epl_if_lst "Expected points left in set if challenge lost"
label variable epl_if_rpl "Expected points left in set if challenge replayed"

do calc_playing_pr.ado

gen byte sample = !doubles & chal_serving != . & miscode == 99 & mtch_chal_pt_win_prob_srv != . & set != . & scoreprob != 1

mata
myGamesSetsPost  = st_data(.,tokens("mtch_chal_score_1 mtch_chal_score_2 mtch_chal_score_3"), "sample")
oppGamesSetsPost = st_data(.,tokens("mtch_opp_score_1 mtch_opp_score_2 mtch_opp_score_3"), "sample")
myGamesSetsCur   = st_data(.,tokens("set1_chal set2_chal set3_chal"), "sample")
oppGamesSetsCur  = st_data(.,tokens("set1_opp set2_opp set3_opp"), "sample")
curSetNum 	 	 = st_data(.,"set","sample")
myPoints  		 = st_data(.,"game_chal", "sample")
oppPoints 		 = st_data(.,"game_opp", "sample")
serving   		 = st_data(.,"chal_serving", "sample")
probs     		 = st_data(.,tokens("mtch_chal_pt_win_prob_srv mtch_chal_pt_win_prob_rcv"), "sample")
st_view(PWMs = ., ., tokens("pwm_if_won pwm_if_lst pwm_if_rpl"), "sample")
st_view(EpointsLeft = ., ., tokens("epl_if_won epl_if_lst epl_if_rpl"), "sample")
st_view(errFlags = ., ., "error_in_pwm", "sample")

stata("gen obsNum = _n")
obsNumData= st_data(.,"obsNum","sample")
st_dropvar("obsNum")

// create a dictionary:
// ARRAY INDEX: 1 2 3 4 ... 14 15 16 ... 29 30 31 ... 39 40 41 ... 49 50 
// POINTS FROM: 0 1 2 3 4 ...  14 15 16 ... 29 30 31 ... 39 40 41 ... 49 50 
// POINTS TO  : 0 . . . . ...  .  1  . ...  .  2  . ...  .  3  . ...  .  4 
dic = J(1,51,.)

dic[51] = (dic[41] = (dic[31] = (dic[16] = (dic[1] = 0) + 1) + 1) + 1) + 1
dic

for(obs=1 ; obs <= rows(PWMs) ; obs++) {
	printf("Working on obs %g\n\n", obsNumData[obs])
	
	// Calculate the number of sets I won and the number of sets opponent won
	errFlag = 0
	mySets = 0
	oppSets = 0
	for(setNum = 1 ; setNum < curSetNum[obs] ; setNum++) {
		if (myGamesSetsPost[obs, setNum] == . | oppGamesSetsPost[obs, setNum] == .) {
			printf("No game info for previous sets, for obs %g\n", obsNumData[obs])
			errFlag = 1
		}
		else if (myGamesSetsPost[obs, setNum] > oppGamesSetsPost[obs, setNum]) {
			mySets++
		}
		else if (myGamesSetsPost[obs, setNum] < oppGamesSetsPost[obs, setNum]) {
			oppSets++
		}
		else {
			printf("Previous set(s) undetermined, for obs %g\n", obsNumData[obs])
			errFlag = 2
		}
	}
	
	errFlags[obs] = errFlag
	if (errFlag) continue
	
	myGames  = myGamesSetsCur[obs, curSetNum[obs]]
	oppGames = oppGamesSetsCur[obs, curSetNum[obs]]
	
	servingLast = serving[obs]
	servingNext = servingLast
	
	// If it's a tiebreaker, next point might have a different server
	if (myGames == 6 & oppGames == 6) {
		if (mod(myPoints[obs] + oppPoints[obs], 2) == 0) {
			servingNext = 1 - servingLast
		}
		myPointsThisObs = myPoints[obs]
		oppPointsThisObs = oppPoints[obs]
	}
	else {
		// if it's not a tiebreaker, translate 15 to 1, 30 to 2, 40 to 3 and 50 to 4
		if (myPoints[obs] > 50) {
			myPointsThisObs = .
			errFlag = 3
		} else
			myPointsThisObs = dic[myPoints[obs]+1]
		
		if (oppPoints[obs] > 50) {
			oppPointsThisObs = .
			errFlag = 3
		}else
			oppPointsThisObs = dic[oppPoints[obs]+1]
	}
	
	errFlags[obs] = errFlag
	
	printf("Current score is %g-%g (sets) %g:%g (games) %g-%g (points)\n",mySets, oppSets, myGames, oppGames, myPointsThisObs, oppPointsThisObs)
	printf("Serving is %g and probability to win a point is %g serving and %g receiving\n\n",servingLast, probs[obs, 1], probs[obs, 2])
	// Probability to win match if point challenged was won
	PWMs[obs, 1] = winMatchProb(mySets, oppSets, myGames, oppGames, ///
								myPointsThisObs+1, oppPointsThisObs, probs[obs, 1], ///
								probs[obs, 2], servingNext)
								
	// Probability to win match if point challenged was lost
	PWMs[obs, 2] = winMatchProb(mySets, oppSets, myGames, oppGames, ///
								myPointsThisObs, oppPointsThisObs+1, probs[obs, 1], ///
								probs[obs, 2], servingNext)
								
	// Probability to win match if point challenged was replayed
	PWMs[obs, 3] = winMatchProb(mySets, oppSets, myGames, oppGames, ///
								myPointsThisObs, oppPointsThisObs, probs[obs, 1], ///
								probs[obs, 2], servingLast)
	if (PWMs[obs,1] == .)
		// stata("pause")
		
	PWMs[obs, .]
	
	// Now, E[points left in set]
	// if point challenged was won
	EpointsLeft[obs, 1] = EpointsLeft(myGames, oppGames, ///
								myPointsThisObs+1, oppPointsThisObs, probs[obs, 1], ///
								probs[obs, 2], servingNext)
								
	// if point challenged was lost
	EpointsLeft[obs, 2] = EpointsLeft(myGames, oppGames, ///
								myPointsThisObs, oppPointsThisObs+1, probs[obs, 1], ///
								probs[obs, 2], servingNext)
								
	// if point challenged was replayed
	EpointsLeft[obs, 3] = EpointsLeft(myGames, oppGames, ///
								myPointsThisObs, oppPointsThisObs, probs[obs, 1], ///
								probs[obs, 2], servingLast)
								
	EpointsLeft[obs, .]
}

end

save point_data_after_importance_round3, replace

/** OLD VERSION (USING THE RECURSIVE PROGRAM) - DO NOT USE UNLESS YOU ARE REALLY BORED
// Run the PWM calculations in chunks of 100 obs
local tot_hunds = ceil(_N/100)*100    // So that the last iteration of the forvalues include the last
                                      // observations (from 1 to 2134, for example, we need the loop to
                                      // go from 1 to 2200)
                                      
forvalues chunk = 1(100)`tot_hunds' {      // FIX THIS TOO TO BE 1(100)`tot_hunds'
	di as input "Working on chunk beginning `chunk'"
	local chunk_end = min(`chunk' + 99, _N)
	calc_winning_prob pwm_if_won pwm_if_lst pwm_if_rpl in `chunk'/`chunk_end' ///
					if chal_serving != . & miscode == 99 & tiebreak != . & mtch_chal_pt_win_prob_srv != . & set != . ///
						, curset(set) ownsets(set?_chal) /// 
						oppsets(set?_opp) points(game_chal game_opp) ///
						pr(mtch_chal_pt_win_prob_srv mtch_chal_pt_win_prob_rcv) ///
						ser(chal_serving) doubles(doubles) tiebreaker(tiebreak)
	
	
}

**/
