// This file prepares the nice challenges (for which we have a full valid state: i.e valid score and server info)
// and saves them as a tab-delimited file.

cd "C:\Users\Roy\Documents\My Dropbox\Tennis\Data\Liran edits\"

local filename "point_data_after_importance_round3"
use "`filename'.dta", clear

// Keep track of which challenge is which by creating an id variable
keep if !doubles & !female
gen challenge_id = _n

// Keep only the nice challenges
keep if pwm_if_won != . & challenge_won != . & distance_mm != .

rename sets_won_chal mysets
rename sets_won_opp  oppsets
rename curset_chal   mygames
rename curset_opp    oppgames
rename game_chal     mypoints
rename game_opp      opppoints
rename chal_serving  serving
rename challenges_left chals_left
rename mtch_chal_pt_win_prob_srv p_win_pt_serve
rename mtch_chal_pt_win_prob_rcv p_win_pt_receive

// Translate tiebreak scores that are higher than 6:6 (or 7:6) to the equivalent 6:6 (or 7:6)
gen orig_mypoints  = mypoints
gen orig_opppoints = opppoints
replace mypoints   = 6 + (orig_mypoints == orig_opppoints + 1) if tiebreak & orig_mypoints > 6 & orig_opppoints > 6 & orig_mypoints != .
replace opppoints  = 6 + (orig_opppoints == orig_mypoints + 1) if tiebreak & orig_mypoints > 6 & orig_opppoints > 6 & orig_mypoints != .

gen chals_in_set_rule = 2 + (tournament_id == 26 | tournament_id == 27)

// Keep only the interesting variables for estimation.
keep challenge_id p_win_pt_serve p_win_pt_receive mysets oppsets mygames oppgames mypoints ///
		opppoints serving chals_left challenge_won distance_mm tiebreak chals_in_set_rule
order challenge_id p_win_pt_serve p_win_pt_receive mysets oppsets mygames oppgames mypoints ///
		opppoints serving chals_left challenge_won distance_mm tiebreak chals_in_set_rule
		
// Translate the 0-15-30-40-50(Adv) scores to 0-1-2-3-4 (while keeping the tiebreaks as they are)
foreach side in my opp {
	gen `side'points_sane = `side'points + (`side'points>30 & !tiebreak)*5 + (`side'points>40 & !tiebreak)*5
	replace `side'points_sane = `side'points_sane / 15 if !tiebreak
	
	bysort tiebreak: tab `side'points `side'points_sane
	drop `side'points
	rename `side'points_sane `side'points
}

outsheet using "..\..\Matlab\challenge_value\\`filename'_for_matlab.txt", replace

